#include "lsjpg.h"

static char *bgcolor, *xgeometry;
static char *file_list_filename;

static struct option long_options[] = {
  {"background-color", 1, 0, 'b'},
  {"no-reading-dialog", 0, 0, 'd'},
  {"list-of-image", 1, 0, 'f'},
  {"geometry", 1, 0, 'g'},
  {"help", 0, 0, 'h'},
  {"index-create", 0, 0, 'i'},
  {"keep-position", 0, 0, 'k'},
  {"light-action", 0, 0, 'l'},
  {"selection-output", 0, 0, 'o'},
  {"pitch", 1, 0, 'p'},
  {"quality", 1, 0, 'q'},
  {"thumbnail-height", 1, 0, 't'},
  {"version", 0, 0, 'v'},
  {"debug", 0, 0, 'D'},
  {"verbose", 0, 0, 'V'},
  {"size", 1, 0, 10},
  {0, 0, 0, 0}
};

int
handleArgs (int argc, char **argv)
{
  extern char *file_list_filename;
  extern int thumb_height, border_width, size_prop_factor, quality;
  extern unsigned status;

  extern char *optarg;
  extern int optind, opterr, optopt;

  int c;

  while (1) {
    int option_index = 0;

    c = getopt_long (argc, argv,
		     "b:dDf:g:hiklop:st:vV", long_options, &option_index);
    if (c == -1)
      break;

    switch (c) {
    case 'b':
      bgcolor = optarg;
      break;

    case 'g':
      xgeometry = optarg;
      break;

    case 'h':
      usage ();
      exit (0);
      break;

    case 'v':
      showVersion ();
      exit (0);
      break;

    case 'V':
      status |= VERBOSE;
      break;

    case 'D':
      status |= DEBUG;
      break;

    case 10:
      size_prop_factor = atoi (optarg);
    case 's':
      status |= SIZE_PROPORTIONAL;
      break;

    case 't':
      thumb_height = atoi (optarg);

      if (thumb_height < 30) {
	fprintf (stderr,
		 "lsjpg: invalid thumbnail size: must 30 < height < 500 (-t %d is invalid)\n",
		 thumb_height);
	thumb_height = 30;
      }
      else if (500 < thumb_height) {
	fprintf (stderr,
		 "lsjpg: invalid thumbnail size: must 30 < height < 500 (-t %d is invalid)\n",
		 thumb_height);
	thumb_height = 500;
      }
      status |= SPECIFY_THUMB_HEIGHT;
      break;

    case 'p':			/* pitch */
      border_width = atoi (optarg);
      break;

    case 'q':
      quality = atoi (optarg);
      break;

    case 'i':
      status |= CREATE_INDEX;
      break;

    case 'd':
      status &= ~DISPLAY_READING_DIALOG;
      break;

    case 'k':
      status |= KEEP_POSITION;
      break;

    case 'l':
      status |= LIGHT_ACTION;
      break;

    case 'o':
      status |= SELECTION_OUTPUT;
      break;

    case 'f':
      status |= READ_FROM_FILE_LIST;
      file_list_filename = optarg;
      break;

    default:
      if (status & VERBOSE) {
	fprintf (stderr, "lsjpg: getopt returned character code 0%o\n", c);
      }
      usage ();
      exit (1);
    }
  }
  return optind;
}

void
usage (void)
{
  char *language = (char *) getenv ("LANG");

  if (!strcmp (language, "ja_JP.ujis"))
    fprintf (stderr, USAGE_MESSAGE_JAPANESE);

  else if (!strcmp (language, "ja_JP.eucJP"))
    fprintf (stderr, USAGE_MESSAGE_JAPANESE);

  else
    fprintf (stderr, USAGE_MESSAGE);
}

void
showVersion (void)
{
  printf (VERSION);
  putchar ('\n');
}

void
setOptionParams (void)
{
  extern unsigned status;
  extern Display *d;

  char *tmpstr;

  status |= DISPLAY_READING_DIALOG;

  if ((tmpstr = XGetDefault (d, "lsjpg", "geometry")))
    xgeometry = tmpstr;

  if ((tmpstr = XGetDefault (d, "lsjpg", "Background")))
    bgcolor = tmpstr;
  else
    bgcolor = BACKGROUND_COLOR;
}

char *
getThumbwinColor (void)
{
  return bgcolor;
}

char *
getThumbwinGeometry (void)
{
  return xgeometry;
}

char *
getFileListFilename (void)
{
  return file_list_filename;
}
