#include "lsjpg.h"

void
handleBigWinEvent (XEvent * event)
{
  extern unsigned status, paint_mode_type;
  extern int xs0, ys0, xs, ys, xe, ye;
  extern GC gc_big, gc_paint, gc_clip;
  extern IMG *image;
  extern XImage *big_image;
  extern Window big_window;
  extern Pixmap big_pmap;
  extern Display *d;

  static char menu_setup_ready = FALSE;
  static char pmode_drawing = FALSE;

  switch (event->type) {
  case Expose:
    if (event->xexpose.count == 0) {
      XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
		 big_image->width, big_image->height, 0, 0);
      status &= ~EXIST_CLIP;
    }
    break;

  case ClientMessage:
    {
      Atom delw;
      XClientMessageEvent *client_event;

      client_event = (XClientMessageEvent *) event;
      delw = XInternAtom (d, "WM_DELETE_WINDOW", FALSE);

      if (client_event->data.l[0] == delw)
	killBigWindow ();
    }
    break;

  case ButtonPress:
    if (event->xbutton.button == Button1) {
      if (status & EXIST_CLIP)
	makeClipRect (xs0, xs, ys0, ys);

      xs0 = xs = event->xbutton.x;
      ys0 = ys = event->xbutton.y;

      if (status & PAINT_MODE) {
	pmode_drawing = TRUE;
	if (paint_mode_type == PMODE_PAINT) {
	  XDrawLine (d, big_window, gc_paint, xs, ys, xs, ys);
	  XDrawLine (d, big_pmap, gc_paint, xs, ys, xs, ys);
	}
      } else {
	makeClipRect (xs0, xs, ys0, ys);
	status |= EXIST_CLIP;
      }
    }
    else if (event->xbutton.button == Button2) {
      if (status & EXIST_CLIP) {
	image = cropImage (image);
	changeWindowSize (image->width, image->height);
	status &= ~EXIST_CLIP;
      }
    }
    else if (event->xbutton.button == Button3) {
      status &= ~EXIST_CLIP;
      if (status & EXIST_MENU) {
	menu_setup_ready = FALSE;
	disposeMenu ();
      }
      else
	menu_setup_ready = TRUE;

      if (status & PAINT_MODE) {
	disposePmenu ();
	menu_setup_ready = FALSE;
      }
    }
    else if (event->xbutton.button == Button4) {
      scrollUp ();
    }
    else if (event->xbutton.button == Button5) {
      scrollDown ();
    }

    break;

  case ButtonRelease:
    if (menu_setup_ready) {
      menuSetup (event);
      menu_setup_ready = FALSE;
    }
    if (status & PAINT_MODE && paint_mode_type != PMODE_PAINT) {
      if (pmode_drawing) {
	switch (paint_mode_type) {
	case PMODE_LINE:
	  XDrawLine (d, big_window, gc_clip, xs0, ys0,
		     event->xbutton.x, event->xbutton.y);
	  XDrawLine (d, big_window, gc_paint, xs0, ys0,
		     event->xbutton.x, event->xbutton.y);
	  XDrawLine (d, big_pmap, gc_paint, xs0, ys0,
		     event->xbutton.x, event->xbutton.y);
	  break;
	case PMODE_RECTANGLE:
	  {
	    int tmp_xs = xs0;
	    int tmp_ys = ys0;
	    int tmp_xe = event->xbutton.x;
	    int tmp_ye = event->xbutton.y;
	    if (tmp_xs > tmp_xe) swap(&tmp_xs, &tmp_xe);
	    if (tmp_ys > tmp_ye) swap(&tmp_ys, &tmp_ye);

	    makeClipRect (xs0, xs, ys0, ys);

	    if (status & PAINT_FILL) {
	      XFillRectangle (d, big_window, gc_paint, tmp_xs, tmp_ys,
			      tmp_xe - tmp_xs, tmp_ye - tmp_ys);
	      XFillRectangle (d, big_pmap, gc_paint, tmp_xs, tmp_ys,
			      tmp_xe - tmp_xs, tmp_ye - tmp_ys);
	    } else {
	      XDrawRectangle (d, big_window, gc_paint, tmp_xs, tmp_ys,
			      tmp_xe - tmp_xs, tmp_ye - tmp_ys);
	      XDrawRectangle (d, big_pmap, gc_paint, tmp_xs, tmp_ys,
			      tmp_xe - tmp_xs, tmp_ye - tmp_ys);
	    }
	  }
	  break;

	case PMODE_CIRCLE:
	  {
	    int tmp_xs = xs0;
	    int tmp_ys = ys0;
	    int tmp_xe = event->xbutton.x;
	    int tmp_ye = event->xbutton.y;
	    if (tmp_xs > tmp_xe) swap(&tmp_xs, &tmp_xe);
	    if (tmp_ys > tmp_ye) swap(&tmp_ys, &tmp_ye);

	    makeClipCircle (xs0, xs, ys0, ys);

	    if (status & PAINT_FILL) {
	      XFillArc (d, big_window, gc_paint, tmp_xs, tmp_ys,
			tmp_xe - tmp_xs, tmp_ye - tmp_ys, 0, 360*64);
	      XFillArc (d, big_pmap, gc_paint, tmp_xs, tmp_ys,
			tmp_xe - tmp_xs, tmp_ye - tmp_ys, 0, 360*64);
	    } else {
	      XDrawArc (d, big_window, gc_paint, tmp_xs, tmp_ys,
			tmp_xe - tmp_xs, tmp_ye - tmp_ys, 0, 360*64);
	      XDrawArc (d, big_pmap, gc_paint, tmp_xs, tmp_ys,
			tmp_xe - tmp_xs, tmp_ye - tmp_ys, 0, 360*64);
	    }
	  }
	  break;

	default:
	  break;
	}
      }
      pmode_drawing = FALSE;
    }
    break;

  case MotionNotify:
    if (status & EXIST_CLIP) {
      /* Draw once at old location (to erase line). */
      makeClipRect (xs0, xs, ys0, ys);

      /* Update end-point and redraw */
      xe = event->xbutton.x;
      ye = event->xbutton.y;
      makeClipRect (xs0, xe, ys0, ye);

    } else if (status & PAINT_MODE && pmode_drawing) {
      if (paint_mode_type == PMODE_PAINT) {
	xe = event->xbutton.x;
	ye = event->xbutton.y;
	XDrawLine (d, big_window, gc_paint, xs, ys, xe, ye);
	XDrawLine (d, big_pmap, gc_paint, xs, ys, xe, ye);
      } else if (paint_mode_type == PMODE_LINE) {
	XDrawLine (d, big_window, gc_clip, xs0, ys0, xs, ys);
	xe = event->xbutton.x;
	ye = event->xbutton.y;
	XDrawLine (d, big_window, gc_clip, xs0, ys0, xe, ye);
      } else if (paint_mode_type == PMODE_RECTANGLE) {
	makeClipRect (xs0, xs, ys0, ys);
	xe = event->xbutton.x;
	ye = event->xbutton.y;
	makeClipRect (xs0, xe, ys0, ye);
      } else if (paint_mode_type == PMODE_CIRCLE) {
	makeClipCircle (xs0, xs, ys0, ys);
	xe = event->xbutton.x;
	ye = event->xbutton.y;
	makeClipCircle (xs0, xe, ys0, ye);
      }
    }
    /* Start point must be changed to current end point */
    xs = xe;
    ys = ye;

  case KeyPress:
    handleBigWinKey (event);
    break;

  default:
    ;
  }
}

void
makeClipRect (int x1, int x2, int y1, int y2)
{
  extern Window big_window;
  extern GC gc_clip;
  extern Display *d;

  /* Make sure positive rectangle */
  if (x2 < x1)
    swap (&x1, &x2);
  if (y2 < y1)
    swap (&y1, &y2);

  XDrawRectangle (d, big_window, gc_clip, x1, y1, x2 - x1, y2 - y1);
}

void
makeClipCircle (int x1, int x2, int y1, int y2)
{
  extern Window big_window;
  extern GC gc_clip;
  extern Display *d;

  /* Make sure positive rectangle */
  if (x2 < x1)
    swap (&x1, &x2);
  if (y2 < y1)
    swap (&y1, &y2);

  XDrawArc (d, big_window, gc_clip, x1, y1, x2 - x1, y2 - y1, 0, 360*64);
}

void
killBigWindow (void)
{
  extern unsigned status;
  extern GC gc_big, gc_clip, gc_paint;
  extern IMG *image;
  extern Window big_window;
  extern XImage *big_image;
  extern Display *d;

  if (status & EXIST_MENU)
    disposeMenu ();
  if (status & PAINT_MODE)
    disposePmenu ();
  XDestroyWindow (d, big_window);
  XFreeGC (d, gc_big);
  XFreeGC (d, gc_clip);
  XFreeGC (d, gc_paint);
  free (image->data);
  free (image);
  free (big_image->data);
  free (big_image);

  status &= ~EXIST_BIG_WINDOW;
  status &= ~EXIST_CLIP;
  if (status & EXIST_PUZZLE) {
    freePcell ();
  }
  status &= ~EXIST_PUZZLE;
  status &= ~CONTROL_STATE;
}

IMG *
cropImage (IMG * image)
{
  extern int xs0, ys0, xe, ye;

  int i, j, k, k2;
  IMG *new = (IMG *) malloc (sizeof (IMG));

  if (image->width < xs0 || image->height < ys0)
    return image;

  if (xe < xs0)
    swap (&xe, &xs0);
  if (ye < ys0)
    swap (&ye, &ys0);
  if (xe > image->width)
    xe = image->width - 1;
  if (ye > image->height)
    ye = image->height - 1;
  if (xs0 < 0)
    xs0 = 0;
  if (ys0 < 0)
    ys0 = 0;

  new->height = ye - ys0 + 1;
  new->width = xe - xs0 + 1;

  new->data = (JSAMPLE *) malloc (new->width * new->height * 3);

  for (i = 0; i < new->height; i++) {
    for (j = 0; j < new->width; j++) {
      k = i * new->width * 3 + j * 3;
      k2 = (i + ys0) * image->width * 3 + (j + xs0) * 3;
      new->data[k] = image->data[k2];
      new->data[k + 1] = image->data[k2 + 1];
      new->data[k + 2] = image->data[k2 + 2];
    }
  }
  free (image->data);
  return new;
}

void
swap (int *a, int *b)
{
  int tmp;

  tmp = *a;
  *a = *b;
  *b = tmp;
}
void
adjustImageToWindowSize (void)
{
  extern Window big_window;
  extern IMG *image;
  extern Display *d;

  XWindowAttributes xwa;

  XGetWindowAttributes (d, big_window, &xwa);

  if (xwa.height != image->height || xwa.width != image->width) {
    if (xwa.width - image->width < xwa.height - image->height)
      resizeBigWindow (xwa.height);
    else
      resizeBigWindow (image->height * xwa.width / image->width);
  }
}
void
bigWindowGoHome (void)
{
  extern unsigned status;
  extern Window big_window;
  extern Display *d;

  Window child;
  XWindowChanges xwc;
  XWindowAttributes xwa;

  if (!(status & EXIST_BIG_WINDOW))
    return;

  XGetWindowAttributes (d, big_window, &xwa);
  XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			 0, 0, &xwa.x, &xwa.y, &child);
  xwc.x = (DisplayWidth (d, DefaultScreen (d)) - xwa.width) / 2;
  xwc.y = (DisplayHeight (d, DefaultScreen (d)) - xwa.height) / 2;

  if (xwc.x < 0)
    xwc.x = 0;
  if (xwc.y < 0)
    xwc.y = 0;
  XConfigureWindow (d, big_window, CWX | CWY, &xwc);
}

void
handleBigWinKey (XEvent * event)
{
  extern unsigned status;
  extern IMG *image;
  extern Display *d;
  extern Window big_window;
  extern int selected_image, key_buffer, n_image;

  Window child;
  char kbuff[16];
  XWindowChanges xwc;
  XWindowAttributes xwa;
  KeySym ks;

  if (XEventsQueued (d, QueuedAfterReading) > 5)
    return;

  XLookupString ((XKeyEvent *) event, kbuff, sizeof (kbuff),
		 &ks, (XComposeStatus *) NULL);

  if (status & CONTROL_STATE) {

    switch (ks) {
    case XK_Delete:
      if (confirmDialog ("DELETE") == CONFIRMED)
	deleteCurrentImageFile ();
      return;

    case XK_s:
      if (confirmDialog ("OVERWRITE") == CONFIRMED) {
	overwriteFile ("BOTH IMAGE");
	status &= ~CONTROL_STATE;
      }
      return;

    case XK_t:
      if (confirmDialog ("THUMBNAIL") == CONFIRMED) {
	overwriteFile ("THUMBNAIL ONLY");
	status &= ~CONTROL_STATE;
      }
      return;

    case XK_a:
      adjustImageToWindowSize ();
      return;

    case XK_Up:
      XGetWindowAttributes (d, big_window, &xwa);
      XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			     0, 0, &xwa.x, &xwa.y, &child);
      xwc.y = xwa.y - 5;
      XConfigureWindow (d, big_window, CWY, &xwc);
      return;

    case XK_Down:
      XGetWindowAttributes (d, big_window, &xwa);
      XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			     0, 0, &xwa.x, &xwa.y, &child);
      xwc.y = xwa.y + 5;
      XConfigureWindow (d, big_window, CWY, &xwc);
      return;

    case XK_Right:
      XGetWindowAttributes (d, big_window, &xwa);
      XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			     0, 0, &xwa.x, &xwa.y, &child);
      xwc.x = xwa.x + 5;
      XConfigureWindow (d, big_window, CWX, &xwc);
      return;

    case XK_Left:
      XGetWindowAttributes (d, big_window, &xwa);
      XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			     0, 0, &xwa.x, &xwa.y, &child);
      xwc.x = xwa.x - 5;
      XConfigureWindow (d, big_window, CWX, &xwc);
      return;

    default:
      ;
    }
    return;
  }
  /* control state END */

  switch (ks) {
  case XK_a:
  case XK_s:
    selectFilter (kbuff, image);
  case XK_q:
    killBigWindow ();
    break;

  case XK_y:
    keen (image);
  case XK_b:
  case XK_e:
  case XK_g:
  case XK_i:
  case XK_j:
  case XK_k:
  case XK_l:
  case XK_m:
  case XK_n:
  case XK_o:
  case XK_r:
  case XK_t:
  case XK_u:
  case XK_v:
  case XK_w:
  case XK_x:
  case XK_z:
  case XK_E:
  case XK_X:
  case XK_Y:
  case XK_Z:
    filterInterface (kbuff);
    break;

  case XK_c:
  case XK_d:
  case XK_f:
  case XK_h:
  case XK_p:
  case XK_P:
  case XK_T:
    selectFilter (kbuff, image);
    break;

  case XK_Home:
    bigWindowGoHome ();
    break;

  case XK_Up:
    selected_image -= key_buffer;
    if (selected_image < 0) {
      selected_image = 0;
      XBell (d, 0);
    }
    displaySelectedImage ();
    break;

  case XK_Down:
    selected_image += key_buffer;
    if (n_image - 1 < selected_image) {
      selected_image = n_image - 1;
      XBell (d, 0);
    }
    displaySelectedImage ();
    break;

  default:
    ;
  }
}

void
filterInterface (char *kbuff)
{
  extern unsigned status;
  extern Display *d;
  extern Visual *visual;
  extern GC gc_big;
  extern XImage *big_image;
  extern Pixmap big_pmap;
  extern Window big_window;
  extern int depth;
  extern IMG *image;

  selectFilter (kbuff, image);
  XDestroyImage (big_image);

  big_image = makeXimage (d, visual, depth, image);

  if (big_pmap)
    XFreePixmap (d, big_pmap);
  big_pmap = XCreatePixmap (d, big_window,
			    big_image->width, big_image->height, depth);
  XPutImage (d, big_pmap, gc_big, big_image, 0, 0, 0, 0,
	     big_image->width, big_image->height);
  XCopyArea (d, big_pmap, big_window, gc_big, 0, 0, big_image->width,
	     big_image->height, 0, 0);
  status &= ~EXIST_CLIP;
}

void
deleteCurrentImageFile (void)
{
  extern unsigned status;
  extern int selected_image;
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];

  char thumb[FILENAME_LENGTH], tmp[FILENAME_LENGTH];

  if (selected_image == -1)
    return;

  if (!ENABLE_OVER_WRITE) {
    fprintf (stderr,
	     "lsjpg: deleterCurrentImageFile: you can *NOT* delete file.\nChange \"lsjpg.h\" of ENABLE_OVER_WRITE.\n");
    return;
  }

  killBigWindow ();

  if (!unlink (dname[selected_image]))
    if (status & VERBOSE)
      fprintf (stderr, "lsjpg: remove file: %s\n", dname[selected_image]);

  strcpy (tmp, dname[selected_image]);
  strcpy (thumb, dirname (tmp));
  strcat (thumb, THUMBDIR_ABS);
  strcat (thumb, basename (dname[selected_image]));

  if (!unlink (thumb))
    if (status & VERBOSE)
      fprintf (stderr, "lsjpg: remove file: %s\n", thumb);

  deleteImageFromList (selected_image);
}
