#include "lsjpg.h"

static GC gc;

int
handleCommonKey (XEvent * event)
{
  extern unsigned status;
  extern Display *d;

  char kbuff[16];
  KeySym ks;

  if (event->type != KeyPress && event->type != KeyRelease)
    return 0;

  XLookupString ((XKeyEvent *) event, kbuff, sizeof (kbuff),
		 &ks, (XComposeStatus *) NULL);

  if (ks == XK_Control_L || ks == XK_Control_R ||
      ks == XK_Alt_L || ks == XK_Alt_R) {
    if (event->type == KeyPress) {
      status |= CONTROL_STATE;
      return 1;
    }
    else if (event->type == KeyRelease) {
      status &= ~CONTROL_STATE;
      return 1;
    }
  }
  if (event->type == KeyRelease)
    return 1;

  if (XEventsQueued (d, QueuedAfterReading) > 5)
    return 0;

  /* Following code is for KeyPress (Not for KeyRelease) */
  if (status & CONTROL_STATE) {
    if (handleControlState (ks))
      return 1;
  }
  else {
    if (handleNonControlState (ks))
      return 1;
  }

  return 0;
}

int
handleControlState (KeySym ks)
{
  extern unsigned status;
  extern int key_buffer, selected_image, n_image;
  extern Display *d;

  switch (ks) {
  case XK_f:
    selected_image = 0;
    displaySelectedImage ();
    return 1;

  case XK_m:
    selected_image = n_image / 2;
    displaySelectedImage ();
    return 1;

  case XK_e:
    selected_image = n_image - 1;
    displaySelectedImage ();
    return 1;

  case XK_n:
    if (n_image - 1 < ++selected_image) {
      selected_image = 0;
      XBell (d, 0);
    }
    displaySelectedImage ();
    return 1;

  case XK_p:
    if (--selected_image < 0) {
      selected_image = n_image - 1;
      XBell (d, 0);
    }
    displaySelectedImage ();
    return 1;

  case XK_N:
  case XK_j:
    selected_image += key_buffer;
    if (n_image - 1 < selected_image) {
      selected_image = n_image - 1;
      XBell (d, 0);
    }
    displaySelectedImage ();
    return 1;

  case XK_P:
    selected_image -= key_buffer;
    if (selected_image < 0) {
      selected_image = 0;
      XBell (d, 0);
    }
    displaySelectedImage ();
    return 1;

  case XK_w:
    if (status & EXIST_BIG_WINDOW)
      killBigWindow ();
    return 1;

  case XK_h:
    bigWindowGoHome ();
    return 1;

  case XK_q:
    /* termination of program */
    exit (0);

  default:
    ;
  }

  return 0;
}

int
handleNonControlState (KeySym ks)
{
  extern int selected_image, n_image, key_buffer;
  extern Display *d;

  switch (ks) {

  case XK_question:
    {
      int r = selected_image;
      while (r == selected_image) {
	r = n_image * (float) random () / RAND_MAX;
      }
      selected_image = r;
      displaySelectedImage ();
    }
    return 1;

  case XK_F:
    fillImageToRootWindow ();
    return 1;

  case XK_Tab:
    setSuitableWinPlace ();
    return 1;

  case XK_BackSpace:
    if (selected_image >= n_image || selected_image < 0)
      selected_image = 0;
    displaySelectedImage ();
    return 1;

  case XK_Right:
  case XK_space:
  case XK_Return:
    if (++selected_image >= n_image) {
      selected_image = 0;
      XBell (d, 0);
    }
    displaySelectedImage ();
    return 1;

  case XK_Left:
    if (--selected_image < 0) {
      selected_image = n_image - 1;
      XBell (d, 0);
    }
    displaySelectedImage ();
    return 1;

  case XK_1:
    key_buffer = 1;
    return 1;
  case XK_2:
    key_buffer = 2;
    return 1;
  case XK_3:
    key_buffer = 3;
    return 1;
  case XK_4:
    key_buffer = 4;
    return 1;
  case XK_5:
    key_buffer = 5;
    return 1;
  case XK_6:
    key_buffer = 6;
    return 1;
  case XK_7:
    key_buffer = 7;
    return 1;
  case XK_8:
    key_buffer = 8;
    return 1;
  case XK_9:
    key_buffer = 9;
    return 9;

  default:
    ;
  }

  return 0;
}

void
fillImageToRootWindow (void)
{
  extern unsigned status;
  extern Display *d;
  extern int depth;
  extern Visual *visual;
  extern Pixmap background_pmap;
  extern Window big_window, thumb_window, menu_window;
  extern int n_image, key_buffer, selected_image;
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];

  int i, x0 = 0, y0 = 0;
  int width, height, target_height, div_height;
  XImage *ximage = NULL;
  IMG *scaled_image = NULL, *image = (IMG *) malloc (sizeof (IMG));

  if (status & EXIST_MENU)
    XUnmapWindow (d, menu_window);
  if (status & EXIST_BIG_WINDOW)
    XUnmapWindow (d, big_window);
  XUnmapWindow (d, thumb_window);

  width = DisplayWidth (d, DefaultScreen (d));
  height = DisplayHeight (d, DefaultScreen (d));

  if (key_buffer < 1)
    div_height = 3;
  else
    div_height = key_buffer;

  target_height = height / div_height;

  if (!(status & EXIST_BACKGROUND)) {
    Colormap default_cmap;
    XColor color;

    background_pmap =
      XCreatePixmap (d, RootWindow (d, 0), width, height, depth);
    default_cmap = DefaultColormap (d, DefaultScreen (d));
    XParseColor (d, default_cmap, BACKGROUND_BACKGROUND, &color);;
    XAllocColor (d, default_cmap, &color);
    gc = XCreateGC (d, background_pmap, 0, 0);
    XSetForeground (d, gc, color.pixel);
    XFillRectangle (d, background_pmap, gc, 0, 0, width, height);
    status |= EXIST_BACKGROUND;
  }

  if (status & VERBOSE)
    fprintf (stderr,
	     "lsjpg: fillImageToRootWindow: putting images to Root-window. Start...\n");

  if (selected_image < 0 || n_image - 1 < selected_image)
    i = 0;
  else
    i = selected_image;

  while (i < n_image) {
    if (readJPEG (dname[i], image) == 0)
      fprintf (stderr,
	       "lsjpg: fillImageToRootWindow: can't find file: %s\n",
	       dname[i]);
    scaled_image = resizeImage (image, target_height);
    windowFrame (scaled_image);
    ximage = makeXimage (d, visual, depth, scaled_image);
    XPutImage (d, background_pmap, gc, ximage,
	       0, 0, x0, y0, ximage->width, ximage->height);
    XSetWindowBackgroundPixmap (d, RootWindow (d, 0), background_pmap);
    XClearWindow (d, RootWindow (d, 0));

    free (image->data);
    free (scaled_image->data);
    free (ximage->data);

    x0 += ximage->width;

    if (width + 100 < x0) {
      i--;
      x0 = 0;
      y0 += target_height;
    }
    if (height - 10 < y0) {
      if (status & VERBOSE)
	fprintf (stderr,
		 "lsjpg: fillImageToRootWindow: Screen size is too small to put all of images.\n");
      break;
    }
    i++;
  }
  free (ximage);
  free (image);
  free (scaled_image);

  XMapWindow (d, thumb_window);
  if (status & EXIST_BIG_WINDOW)
    XMapWindow (d, big_window);

  if (status & VERBOSE)
    fprintf (stderr, "lsjpg: fillImageToRootWindow: ...done\n");
}

void
addToRootWindow (IMG * image)
{
  extern unsigned status;
  extern int depth;
  extern Display *d;
  extern XImage *big_image;
  extern Pixmap background_pmap;
  extern Window big_window;

  Window child;
  XWindowAttributes xwa;

  XGetWindowAttributes (d, big_window, &xwa);
  XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			 0, 0, &xwa.x, &xwa.y, &child);

  if (!(status & EXIST_BACKGROUND)) {
    int width, height;
    int format;
    unsigned long length, after;
    unsigned char *data;
    Atom prop, type;
    Colormap default_cmap;
    XColor color;

    width = DisplayWidth (d, DefaultScreen (d));
    height = DisplayHeight (d, DefaultScreen (d));

    background_pmap =
      XCreatePixmap (d, RootWindow (d, 0), width, height, depth);
    default_cmap = DefaultColormap (d, DefaultScreen (d));
    XParseColor (d, default_cmap, BACKGROUND_BACKGROUND, &color);;
    XAllocColor (d, default_cmap, &color);
    gc = XCreateGC (d, background_pmap, 0, 0);
    XSetForeground (d, gc, color.pixel);
    XFillRectangle (d, background_pmap, gc, 0, 0, width, height);

    prop = XInternAtom (d, "_XROOTPMAP_ID", True);

    if (prop != None) {
      XGetWindowProperty (d, RootWindow (d, 0), prop, 0L, 1L,
			  False, AnyPropertyType,
			  &type, &format, &length, &after, &data);
      if (type == XA_PIXMAP) {
	Pixmap p;
	unsigned long GC_mask;
	XGCValues xGCv;

	p = *((Pixmap *) data);
	xGCv.fill_style = FillTiled;
	xGCv.tile = p;
	GC_mask = GCFillStyle | GCTile;
	XChangeGC (d, gc, GC_mask, &xGCv);
      }
    }
    XFillRectangle (d, background_pmap, gc, 0, 0, width, height);
    XPutImage (d, background_pmap, gc, big_image, 0, 0, xwa.x, xwa.y,
	       big_image->width, big_image->height);
    XSetWindowBackgroundPixmap (d, RootWindow (d, 0), background_pmap);
    XClearWindow (d, RootWindow (d, 0));
    status |= EXIST_BACKGROUND;

  }
  else {
    XPutImage (d, background_pmap, gc, big_image, 0, 0, xwa.x, xwa.y,
	       big_image->width, big_image->height);
    XSetWindowBackgroundPixmap (d, RootWindow (d, 0), background_pmap);
    XClearWindow (d, RootWindow (d, 0));
  }
}
