#include "lsjpg.h"
#include <math.h>

void
selectFilter (char *selection, IMG * image)
{
  extern unsigned status;

  switch (*selection) {
  case 'a':
    addToRootWindow (image);
    break;
  case 'b':
    blackWhite (image);
    break;
  case 'c':
    clockwiseRotate (image);
    changeWindowSize (image->width, image->height);
    break;
  case 'd':			/* Double size */
    resizeBigWindow (image->height * DOUBLE_FACTOR);
    break;
  case 'e':
    edgeEnhance (image);
    break;
  case 'f':
    fullSize ();
    break;
  case 'g':
    grayScale (image);
    break;
  case 'h':			/* Half size */
    resizeBigWindow (image->height / HALF_FACTOR);
    break;
  case 'i':
    ifunction (image);
    break;
  case 'j':
    jfunction (image);
    break;
  case 'k':
    keen (image);
    break;
  case 'l':
    lightUp (image);
    break;
  case 'm':
    median (image);
    break;
  case 'n':
    nfunction (image);
    break;
  case 'o':
    opposite (image);
    break;
  case 'P':
    status ^= PUZZLE_WITH_BORDER;
  case 'p':
    puzzle (image);
    break;
  case 'r':
    reverseColor (image);
    break;
  case 's':
    setToRootWindow (image);
    break;
  case 't':
    thinner (image);
    break;
  case 'u':
    unSharpMask (image);
    break;
  case 'v':
    vibrate (image);
    break;
  case 'w':
    windowFrame (image);
    break;
  case 'x':
    xyConv ("x", image);
    break;
  case 'y':
    xyConv ("y", image);
    break;
  case 'z':
    zoom (image);
    break;
  case 'X':
    XZanim ("X", image);
    break;
  case 'Y':
    YFunction (image);
    break;
  case 'Z':
    XZanim ("Z", image);
    break;
  case 'T':
    if (!(status & SELECTION_OUTPUT))
      textOutput (image);
    break;
  case 'E':
    edgeDetect (image);
    break;

  default:
    ;
  }
}

void
blackWhite (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;
  Window statusbar;
  XWindowAttributes xwa;

  int i, j, k;
  int grayscale, threshold = 128, sum = 0;
  int freq[256];

  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0, xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));

  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);

  for (i = 0; i < 256; i++)
    freq[i] = 0;

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      grayscale = (image->data[i * image->width * 3 + j * 3] +
		   image->data[i * image->width * 3 + j * 3 + 1] +
		   image->data[i * image->width * 3 + j * 3 + 2]) / 3;
      for (k = 0; k < 256; k++)
	if (grayscale <= k)
	  break;
      freq[k]++;
    }
    XFillRectangle (d, statusbar, gc_big, 2, 1,
		    (xwa.width - 6) * i / (image->height * 2), 7);
    XFlush (d);
  }

  for (i = 0; i < 256; i++) {
    sum += freq[i];
    if (sum > image->height * image->width / 2) {
      threshold = i;
      break;
    }
  }

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      k = i * image->width * 3 + j * 3;
      grayscale = (image->data[k]
		   + image->data[k + 1] + image->data[k + 2]) / 3;
      if (grayscale < threshold) {
	image->data[k] = 0;
	image->data[k + 1] = 0;
	image->data[k + 2] = 0;

      }
      else {
	image->data[k] = 255;
	image->data[k + 1] = 255;
	image->data[k + 2] = 255;
      }
    }
    XFillRectangle
      (d, statusbar, gc_big, 2, 1,
       (xwa.width - 6) * (i + image->height) / (image->height * 2), 7);
    XFlush (d);
  }
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
}

void
clockwiseRotate (IMG * image)
{
  extern int original_image_size;

  int i, j, k;
  int *tmp = (int *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->width; i++) {
    for (j = image->height - 1; j >= 0; j--) {
      k = j * image->width * 3 + i * 3;
      *(tmp++) = image->data[k];
      *(tmp++) = image->data[k + 1];
      *(tmp++) = image->data[k + 2];
    }
  }
  tmp -= image->height * image->width * 3;

  for (i = 0; i < image->height * image->width * 3; i++)
    image->data[i] = *(tmp++);

  free (tmp - image->height * image->width * 3);
  swap (&image->height, &image->width);
  original_image_size = image->height;
}

void
edgeEnhance (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;
  Window statusbar;
  XWindowAttributes xwa;

  char clr;
  int i, j, k;
  int a, *tmp;
  double laplacian, ratio = ENHANCE_RATIO;

  /* status bar */
  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));


  tmp = (int *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];


  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);

  for (i = 1; i < image->height - 1; i++) {
    for (j = 1; j < image->width - 1; j++) {
      k = i * image->width * 3 + j * 3;
      for (clr = 0; clr < 3; clr++) {
	laplacian = tmp[k - 3 * image->width + clr]
	  + tmp[k + 3 * image->width + clr]
	  + tmp[k - 3 + clr] + tmp[k + 3 + clr] - 4 * tmp[k + clr];

	a = (int) (tmp[k + clr] - ratio * laplacian);
	if (a < 0)
	  a = 0;
	if (255 < a)
	  a = 255;
	image->data[k + clr] = a;
      }
    }
    XFillRectangle (d, statusbar, gc_big, 2, 1,
		    (xwa.width - 6) * (i + 1) / (image->height - 1), 7);
    XFlush (d);
  }
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
  free (tmp);
}

void
fullSize (void)
{
  extern Display *d;
  extern XImage *big_image;
  extern Window big_window;
  extern IMG *image;

  int dispW = DisplayWidth (d, DefaultScreen (d));
  int dispH = DisplayHeight (d, DefaultScreen (d));
  float screen_aspect, image_aspect;
  IMG *tmp;
  XWindowChanges xwc;
  XWindowAttributes xwa;
  Window child;

  screen_aspect = (float) dispH / dispW;
  image_aspect = (float) big_image->height / big_image->width;

  XGetWindowAttributes (d, big_window, &xwa);
  XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			 0, 0, &xwa.x, &xwa.y, &child);

  if (image_aspect < screen_aspect)	/* Landscape type */
    xwc.height =
      big_image->height * (dispW - 2 * FULL_SIZE_MARGIN) / big_image->width;
  else				/* Portrait type */
    xwc.height = dispH - 8 - 2 * FULL_SIZE_MARGIN;

  if (xwc.height == big_image->height)
    return;

  XStoreName (d, big_window, "Full screen...");
  XFlush (d);

  tmp = resizeImage (image, xwc.height);

  free (image->data);
  image->data = tmp->data;
  image->width = tmp->width;
  image->height = tmp->height;

  xwc.width = image->width;

  /*
  xwc.x = (dispW - xwc.width) / 2;
  xwc.y = (dispH - xwc.height) / 2;
  XConfigureWindow (d, big_window, CWX | CWY | CWWidth | CWHeight, &xwc);
  */
  XConfigureWindow (d, big_window, CWWidth | CWHeight, &xwc);

  windowConfig ();
}

void
grayScale (IMG * image)
{
  int grayscale, i, j;

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      grayscale = (image->data[i * image->width * 3 + j * 3]
		   + image->data[i * image->width * 3 + j * 3 + 1]
		   + image->data[i * image->width * 3 + j * 3 + 2]) / 3;

      image->data[i * image->width * 3 + j * 3]
	= image->data[i * image->width * 3 + j * 3 + 1]
	= image->data[i * image->width * 3 + j * 3 + 2] = grayscale;
    }
  }
}

void
ifunction (IMG * image)
{
  int i;
  int gtable[256];

  for (i = 0; i < 128; i++)
    gtable[i] = 127 * pow ((float) i / 128, 1.2);
  for (i = 128; i < 256; i++)
    gtable[i] = 127 * pow ((float) (i - 128) / 128, 0.8) + 128;

  for (i = 0; i < image->width * image->height * 3; i++)
    image->data[i] = gtable[image->data[i]];
}

void
jfunction (IMG * image)
{
  int i;
  int gtable[256];

  for (i = 0; i < 128; i++)
    gtable[i] = 127 * pow ((float) i / 128, 0.5);
  for (i = 128; i < 256; i++)

    gtable[i] = 127 * pow ((float) (i - 128) / 128, 1.2) + 128;

  for (i = 0; i < image->width * image->height * 3; i++)
    image->data[i] = gtable[image->data[i]];
}

void
keen (IMG * image)
{
  int i;
  int gtable[256];

  for (i = 0; i < 256; i++)
    gtable[i] = 255 * pow ((float) i / 256, 1.2);

  for (i = 0; i < image->width * image->height * 3; i++)
    image->data[i] = gtable[image->data[i]];
}

void
lightUp (IMG * image)
{
  int i;
  int gtable[256];

  for (i = 0; i < 256; i++)
    gtable[i] = 255 * pow ((float) i / 256, 0.8);

  for (i = 0; i < image->width * image->height * 3; i++)
    image->data[i] = gtable[image->data[i]];
}

void
median (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;
  Window statusbar;
  XWindowAttributes xwa;

  int a_, b_, c_, d_, e_, f_, g_, h_, i_;
  int ii, jj, n, *tmp;
  int width = image->width, height = image->height;

  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));

  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);

  n = width * height * 3;
  tmp = (int *) malloc (sizeof (int) * n);

  for (ii = 0; ii < n; ii++)
    tmp[ii] = image->data[ii];

  for (ii = 1; ii < height - 1; ii++) {
    for (jj = 1; jj < width - 1; jj++) {
      e_ = ii * width * 3 + jj * 3;

      a_ = e_ - 3 * width - 3;
      b_ = e_ - 3 * width;
      c_ = e_ - 3 * width + 3;
      d_ = e_ - 3;
      f_ = e_ + 3;
      g_ = e_ + 3 * width - 3;
      h_ = e_ + 3 * width;
      i_ = e_ + 3 * width + 3;

      image->data[e_] =
	cellMdn (tmp[a_], tmp[b_], tmp[c_],
		 tmp[d_], tmp[e_], tmp[f_], tmp[g_], tmp[h_], tmp[i_]);
      image->data[e_ + 1] =
	cellMdn (tmp[a_ + 1], tmp[b_ + 1], tmp[c_ + 1],
		 tmp[d_ + 1], tmp[e_ + 1], tmp[f_ + 1],
		 tmp[g_ + 1], tmp[h_ + 1], tmp[i_ + 1]);
      image->data[e_ + 2] =
	cellMdn (tmp[a_ + 2], tmp[b_ + 2], tmp[c_ + 2],
		 tmp[d_ + 2], tmp[e_ + 2], tmp[f_ + 2],
		 tmp[g_ + 2], tmp[h_ + 2], tmp[i_ + 2]);
    }
    XFillRectangle (d, statusbar, gc_big, 2, 1,
		    (xwa.width - 6) * (ii + 1) / (height - 1), 7);
    XFlush (d);
  }
  free (tmp);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
}

int
cellMdn (int a, int b, int c, int d, int e, int f, int g, int h, int i)
{
  int m, n, tmp;
  int p[9];

  p[0] = a;
  p[1] = b;
  p[2] = c;
  p[3] = d;
  p[4] = e;
  p[5] = f;
  p[6] = g;
  p[7] = h;
  p[8] = i;

  for (m = 0; m < 8; m++) {
    for (n = m + 1; n < 9; n++) {
      if (p[m] > p[n]) {
	tmp = p[m];
	p[m] = p[n];
	p[n] = tmp;
      }
    }
  }
  return p[4];
}

void
nfunction (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;
  Window statusbar;
  XWindowAttributes xwa;

  char clr;
  int *tmp;
  int i, j, k;
  double target;

  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));

  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);


  tmp = (int *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];

  for (i = 1; i < image->height - 1; i++) {
    for (j = 1; j < image->width - 1; j++) {
      k = i * image->width * 3 + j * 3;
      for (clr = 0; clr < 3; clr++) {
	target = (tmp[k + clr - 3 * image->width - 3]
		  + 2 * tmp[k + clr - 3 * image->width]
		  + 1 * tmp[k + clr - 3 * image->width + 3]
		  + 2 * tmp[k + clr - 3]
		  + 3 * tmp[k + clr]
		  + 2 * tmp[k + clr + 3]
		  + 1 * tmp[k + clr + 3 * image->width - 3]
		  + 2 * tmp[k + clr + 3 * image->width]
		  + 1 * tmp[k + clr + 3 * image->width + 3]) / 15;
	if (target > 255)
	  target = 255;
	if (target < 0)
	  target = 0;
	image->data[k + clr] = target;
      }
    }
    XFillRectangle (d, statusbar, gc_big, 2,
		    1, (xwa.width - 6) * (i + 1) / (image->height - 1), 7);
    XFlush (d);
  }
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
  free (tmp);
}

void
opposite (IMG * image)
{
  char *tmp, clr;
  int i, j, k, m;


  tmp = (char *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      k = i * image->width * 3 + j * 3;
      for (clr = 0; clr < 3; clr++) {
	m = i * image->width * 3 + (image->width - j - 1) * 3;
	image->data[k + clr] = tmp[m + clr];
      }
    }
  }
  free (tmp);
}

void
edgeDetect (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;

  char clr;
  int i, j, k;
  int a, *tmp;

  Window statusbar;
  XWindowAttributes xwa;

  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));


  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);


  tmp = (int *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];


  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);

  for (i = 1; i < image->height - 1; i++) {
    for (j = 1; j < image->width - 1; j++) {
      k = i * image->width * 3 + j * 3;
      for (clr = 0; clr < 3; clr++) {
	a = -tmp[k - 3 * image->width - 3 + clr]
	  - tmp[k - 3 * image->width + clr]
	  - tmp[k - 3 * image->width + 3 + clr]
	  - tmp[k - 3 + clr]
	  + tmp[k + clr] * 8 - tmp[k + 3 + clr]
	  - tmp[k + 3 * image->width - 3 + clr]
	  - tmp[k + 3 * image->width + clr]
	  - tmp[k + 3 * image->width + 3 + clr];
	if (a < 0)
	  a = 0;
	if (255 < a)
	  a = 255;
	image->data[k + clr] = ~a;
      }
    }
    XFillRectangle (d, statusbar, gc_big, 2,
		    1, (xwa.width - 6) * (i + 1) / (image->height - 1), 7);
    XFlush (d);
  }
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
  free (tmp);
}

void
reverseColor (IMG * image)
{
  int k, i, j;

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      k = i * image->width * 3 + j * 3;
      image->data[k] = ~image->data[k];
      image->data[k + 1] = ~image->data[k + 1];
      image->data[k + 2] = ~image->data[k + 2];
    }
  }
}
void
setToRootWindow (IMG * image)
{
  extern int depth;
  extern Display *d;
  extern XImage *big_image;

  Pixmap pmap;
  GC gc;

  gc = XCreateGC (d, RootWindow (d, 0), 0, 0);
  pmap = XCreatePixmap (d, RootWindow (d, 0),
			big_image->width, big_image->height, depth);
  XPutImage (d, pmap, gc, big_image, 0, 0, 0, 0,
	     big_image->width, big_image->height);
  XSetWindowBackgroundPixmap (d, RootWindow (d, 0), pmap);
  XClearWindow (d, RootWindow (d, 0));
  XFreePixmap (d, pmap);
}

void
thinner (IMG * image)
{
  char *tmp, clr;
  int i, j, k, m, j2;

  tmp = (char *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      k = i * image->width * 3 + j * 3;
      j2 = (image->width - 1) *
	((1 +sin (M_PI * (float) (j - image->width / 2) / image->width)) / 2);
      m = i * image->width * 3 + j2 * 3;
      for (clr = 0; clr < 3; clr++)
	image->data[k + clr] = tmp[m + clr];
    }
  }
  free (tmp);
}

void
textOutput (IMG * image)
{
  int i, j, shift, pixel, new_width, new_height;
  double f;

  new_width = TEXT_OUTPUT_WIDTH;
  new_height = image->height * ((float) new_width / image->width);

  f = (double) image->height / new_height;

  for (i = 0; i < new_height; i += 2) {
    for (j = 0; j < new_width; j++) {
      shift = 3 * ((int) (f * j) + image->width * (int) (f * i));
      pixel =
	(*(image->data + shift) +
	 *(image->data + shift + 1) + *(image->data + shift + 2)) / 3;
      if (pixel < 32)
	printf ("@");
      else if (pixel < 64)
	printf ("*");
      else if (pixel < 96)
	printf ("%%");
      else if (pixel < 128)
	printf ("O");
      else if (pixel < 160)
	printf ("+");
      else if (pixel < 192)
	printf ("-");
      else if (pixel < 224)
	printf (".");
      else
	printf (" ");
    }
    printf ("\n");
  }
}

void
unSharpMask (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;
  Window statusbar;
  XWindowAttributes xwa;

  char clr;
  int i, j, k;
  int *tmp;
  double target, f = UNSHARP_FACTOR;

  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));

  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);


  tmp = (int *) malloc (sizeof (int) * image->height * image->width * 3);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];

  for (i = 1; i < image->height - 1; i++) {
    for (j = 1; j < image->width - 1; j++) {
      k = i * image->width * 3 + j * 3;
      for (clr = 0; clr < 3; clr++) {
	target = -tmp[k + clr - 3 * image->width - 3]
	  - tmp[k + clr - 3 * image->width]
	  - tmp[k + clr - 3 * image->width + 3]
	  - tmp[k + clr - 3]
	  + tmp[k + clr] * (9 * f - 1) - tmp[k + clr + 3]
	  - tmp[k + clr + 3 * image->width - 3]
	  - tmp[k + clr + 3 * image->width]
	  - tmp[k + clr + 3 * image->width + 3];

	if (target > 255)
	  target = 255;
	if (target < 0)
	  target = 0;
	image->data[k + clr] = target;
      }
    }
    XFillRectangle (d, statusbar, gc_big, 2,
		    1, (xwa.width - 6) * (i + 1) / (image->height - 1), 7);
    XFlush (d);
  }
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
  free (tmp);
}

void
vibrate (IMG * image)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_big;
  Window statusbar;
  XWindowAttributes xwa;

  char clr;
  char *tmp = (char *) malloc (sizeof (int) * image->height * image->width);
  int i, i2, j, j2, k, k2;

  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_watch));

  XDrawRectangle (d, statusbar, gc_big, 2, 1, xwa.width - 6, 7);

  for (i = 0; i < image->height * image->width * 3; i++)
    tmp[i] = image->data[i];

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      k = i * image->width * 3 + j * 3;
      j2 = j + random () / (RAND_MAX / 3) - 1;
      i2 = i + random () / (RAND_MAX / 3) - 1;
      if (j < 0)
	j = 0;
      if (image->width - 1 < j)
	j = image->width;
      k2 = i2 * image->width * 3 + j2 * 3;
      if (k2 < 0)
	k2 = 0;
      if (image->height * image->width * 3 - 3 < k2)
	k2 = image->height * image->width * 3 - 3;
      for (clr = 0; clr < 3; clr++) {
	image->data[k + clr] = tmp[k2 + clr];
      }
    }
    XFillRectangle (d, statusbar, gc_big, 2,
		    1, (xwa.width - 6) * (i + 1) / image->height, 7);
    XFlush (d);
  }
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);
  free (tmp);
}

void
windowFrame (IMG * image)
{
  extern Display *d;
  char *tmp, clr, border, red, green, blue;
  int i, j, k;

  Colormap default_cmap;
  XColor color;

  default_cmap = DefaultColormap (d, DefaultScreen (d));
  XParseColor (d, default_cmap, WINDOW_FRAME_BACKGROUND, &color);
  XAllocColor (d, default_cmap, &color);
  red = 256 * color.red / 65536;
  green = 256 * color.green / 65536;
  blue = 256 * color.blue / 65536;

  tmp = (char *) malloc (sizeof (int) * image->width * image->height * 3);
  for (i = 0; i < image->width * image->height * 3; i++)
    tmp[i] = image->data[i];

  border = 0.05 * image->height;
  if (0.05 * image->width < border)
    border = 0.05 * image->width;

  for (i = 0; i < image->height; i++) {
    for (j = 0; j < image->width; j++) {
      k = i * image->width * 3 + j * 3;

      if (image->height - border < i && i < image->height - border + 14 &&
	  image->width - border < j && j < image->width - border + 14) {
	if (i < image->height - border + 2 && j < image->width - border + 2) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 20;
	}
	else if (i < image->height - border + 4 &&
		 j < image->width - border + 4) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 40;
	}
	else if (i < image->height - border + 6 &&
		 j < image->width - border + 6) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 60;
	}
	else if (i < image->height - border + 8 &&
		 j < image->width - border + 8) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 80;
	}
	else if (i < image->height - border + 10 &&
		 j < image->width - border + 10) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 100;
	}
	else if (i < image->height - border + 12 &&
		 j < image->width - border + 12) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 120;
	}
	else {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 140;
	}
      }
      else if (image->height - border < i
	       && i < image->height - border + 14 && border + 7 < j
	       && j < image->width - border + 14) {
	if (i < image->height - border + 2) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 20;
	}
	else if (i < image->height - border + 4) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 40;
	}
	else if (i < image->height - border + 6) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 60;
	}
	else if (i < image->height - border + 8) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 80;
	}
	else if (i < image->height - border + 10) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 100;
	}
	else if (i < image->height - border + 12) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 120;
	}
	else {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 140;
	}
      }
      else if (border + 7 < i && i < image->height - border + 14 &&
	       image->width - border < j && j < image->width - border + 14) {
	if (j < image->width - border + 2) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 20;
	}
	else if (j < image->width - border + 4) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 40;
	}
	else if (j < image->width - border + 6) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 60;
	}
	else if (j < image->width - border + 8) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 80;
	}
	else if (j < image->width - border + 10) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 100;
	}
	else if (j < image->width - border + 12) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 120;
	}
	else {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = 140;
	}
      }
      else if (i < border || image->height - border < i ||
	       j < border || image->width - border < j) {
	if (USE_ADOMACHIC_BACKGROUND) {
	  for (clr = 0; clr < 3; clr++)
	    image->data[k + clr] = random () / (RAND_MAX / 256);
	}
	else {
	  image->data[k + 0] = red;
	  image->data[k + 1] = green;
	  image->data[k + 2] = blue;
	}
      }
      else {
	for (clr = 0; clr < 3; clr++)
	  image->data[k + clr] = tmp[k + clr];
      }
    }
  }
  free (tmp);
}

void
resizeBigWindow (int h)
{
  extern Display *d;
  extern Window big_window;
  extern IMG *image;

  XWindowChanges xwc;
  IMG *tmp;

  if (h < 10 || 1280 < h)
    return;

  XStoreName (d, big_window, "Adjusting image size...");

  tmp = resizeImage (image, h);
  free (image->data);
  image->data = tmp->data;
  image->width = tmp->width;
  image->height = tmp->height;

  xwc.height = image->height;
  xwc.width = image->width;

  XConfigureWindow (d, big_window, CWWidth | CWHeight, &xwc);

  windowConfig ();
}

void
changeWindowSize (int w, int h)
{
  extern Display *d;
  extern Display *d;
  extern Window big_window;

  Window child;
  XWindowAttributes xwa;
  XWindowChanges xwc;

  XGetWindowAttributes (d, big_window, &xwa);
  XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			 0, 0, &xwa.x, &xwa.y, &child);
  xwc.x = xwa.x;
  xwc.y = xwa.y;

  if (xwc.x < 0)
    xwc.x = 0;
  if (xwc.y < 0)
    xwc.y = 0;

  xwc.height = h;
  xwc.width = w;

  XConfigureWindow (d, big_window, CWX | CWY | CWWidth | CWHeight, &xwc);

  windowConfig ();
}
void
windowConfig (void)
{
  extern char fname[MAX_IMAGE][FILENAME_LENGTH];
  extern int selected_image;
  extern int original_image_size;
  extern int depth;
  extern Display *d;
  extern Visual *visual;
  extern Window big_window;
  extern GC gc_big;
  extern Pixmap big_pmap;
  extern XImage *big_image;
  extern IMG *image;

  char buff[FILENAME_LENGTH];

  XDestroyImage (big_image);
  big_image = makeXimage (d, visual, depth, image);

  if (big_pmap)
    XFreePixmap (d, big_pmap);
  big_pmap = XCreatePixmap (d, big_window,
			    big_image->width, big_image->height, depth);
  XPutImage (d, big_pmap, gc_big, big_image, 0, 0, 0,
	     0, big_image->width, big_image->height);
  XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
	     big_image->width, big_image->height, 0, 0);
  sprintf (buff, "%s: %dx%d(%d%%)",
	   (char *) (fname + selected_image),
	   big_image->width, big_image->height,
	   (int) (100 *
		  ((float) big_image->height / original_image_size) + 0.5));
  XStoreName (d, big_window, buff);
}

void
YFunction (IMG * image)
{
  extern int depth;
  extern Display *d;
  extern Visual *visual;
  extern GC gc_big;
  extern Window big_window;

  int i, j, k, p, bar = 0;
  int rnd[256];
  XImage *xim;
  XEvent event;
  XWindowAttributes xwa;
  XClientMessageEvent *client_event;
  Window statusbar;
  Atom delw;
  XColor color;
  Colormap default_cmap;
  XGCValues xGCv;
  GC gc;
  unsigned long fgpix;

  /* status bar */
  default_cmap = DefaultColormap (d, DefaultScreen (d));
  XParseColor (d, default_cmap, "red", &color);
  XAllocColor (d, default_cmap, &color);
  fgpix = color.pixel;
  xGCv.foreground = fgpix;
  XGetWindowAttributes (d, big_window, &xwa);

  statusbar =
    XCreateSimpleWindow (d, big_window, 0, 0,
			 xwa.width, 10, 1,
			 BlackPixel (d, 0), WhitePixel (d, 0));
  gc = XCreateGC (d, statusbar, GCForeground, &xGCv);
  XMapSubwindows (d, big_window);
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_gumby));
  delw = XInternAtom (d, "WM_DELETE_WINDOW", FALSE);

  XStoreName (d, big_window, "Button-3 to stop");

  while (1) {
    /* DONE? */
    if (XEventsQueued (d, QueuedAfterReading)) {
      XNextEvent (d, &event);
      client_event = (XClientMessageEvent *) & event;

      if ((event.type == ButtonPress && event.xbutton.button == Button3)
	  || (event.type == ClientMessage && client_event->data.l[0] == delw)) {
	/* exit while(1) loop */
	goto DONE;
      }
    }

    XDrawRectangle (d, statusbar, gc, 2, 1, xwa.width - 6, 7);

    for (i = 0; i < 256; i++) {
      rnd[i] = i + (int) (11 * (float) random () / RAND_MAX) - 5;
      if (rnd[i] < 0)
	rnd[i] = 0;
      if (255 < rnd[i])
	rnd[i] = 255;
    }

    for (i = 0; i < image->height; i++) {
      for (j = 0; j < image->width; j++) {
	for (k = 0; k < 3; k++) {
	  p = i * image->width * 3 + j * 3 + k;
	  image->data[p] = rnd[image->data[p]];
	}
      }
    }

    xim = makeXimage (d, visual, depth, image);
    XPutImage (d, big_window, gc_big, xim, 0, 0,
	       0, 0, xim->width, xim->height);
    XDestroyImage (xim);

    XFillRectangle (d, statusbar, gc, 2, 1, ++bar, 7);
    if (xwa.width - 6 < bar) {
      bar = 0;
      XClearWindow (d, statusbar);
    }
  }

DONE:
  XStoreName (d, big_window, "DONE");

  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
  XDestroySubwindows (d, big_window);

  XMapRaised (d, big_window);
}

void
zoom (IMG * image)
{
  extern int xs0, ys0;

  int i, j, w, h, n, xc, yc, *tmp;

  w = image->width;
  h = image->height;
  n = w * h * 3;
  tmp = (int *) malloc (sizeof (int) * n);

  if (xs0 < 1 || ys0 < 1) {
    xc = w / 2;
    yc = h / 4;
  }
  else {
    xc = xs0;
    yc = ys0;
  }

  for (i = 0; i < n; i++)
    tmp[i] = image->data[i];

  for (i = 0; i < h; i++) {
    for (j = 0; j < w; j++) {
      image->data[i * w * 3 + j * 3]
	= tmp[((i - yc) / 2 + yc) * w * 3 + ((j - xc) / 2 + xc) * 3];
      image->data[i * w * 3 + j * 3 + 1]
	= tmp[((i - yc) / 2 + yc) * w * 3 + ((j - xc) / 2 + xc) * 3 + 1];
      image->data[i * w * 3 + j * 3 + 2]
	= tmp[((i - yc) / 2 + yc) * w * 3 + ((j - xc) / 2 + xc) * 3 + 2];
    }
  }
  free (tmp);
}
