#include "lsjpg.h"

/* Message */
#define NOSUCH_FILE     "%s: %s: No such file or directory\n"
#define IGNORE_SUBS     "\nsubdirectory in %s are ignored\n"
#define TOOMANY_FILES   "%s: too many files: First %d images to display\n"
#define FILE_OPEN_ERROR "%s: error opening file: %s\n"
#define NOJPEG          "%s: No JPEG file found. Is there any JPEG file?\n"

int
getImages (int optind, int argc, char **argv)
{
  extern unsigned status;
  extern int depth;
  extern Display *d;
  extern Visual *visual;
  extern XImage *thumb_image[MAX_IMAGE];
  extern char fname[MAX_IMAGE][FILENAME_LENGTH];
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];

  char direcname[FILENAME_LENGTH], *file_list;
  char direcname_xvp[FILENAME_LENGTH];
  int i, j, start_index, n_image, n_file_list;
  int dialog_width, dialog_height;
  IMG *image = (IMG *) malloc (sizeof (IMG));
  Window reading_dialog, textarea;
  GC gc;

  if (argc - optind == 0) {
    start_index = optind - 1;
    argv[start_index] = ".";
  }
  else {
    start_index = optind;
    if (argc - optind == 1) {
      struct stat statinfo;
      stat (argv[optind], &statinfo);
      if (!S_ISDIR (statinfo.st_mode))
	status &= ~DISPLAY_READING_DIALOG;
    }
  }

  if (status & DISPLAY_READING_DIALOG)
    setupReadingDialog (&reading_dialog, &textarea,
			&gc, &dialog_width, &dialog_height);

  file_list = malloc (MAX_IMAGE * FILENAME_LENGTH);

  if (status & READ_FROM_FILE_LIST)
    n_file_list = getFileListFromFile (file_list);
  else
    n_file_list = getFileList (file_list, argc, argv, start_index);

  for (i = 0, j = 0; i < n_file_list; i++) {
    int k, path_specify = FALSE;

    for (k = 0; k < strlen (file_list + i * FILENAME_LENGTH); k++) {
      if (*(file_list + i * FILENAME_LENGTH + k) == '/') {
	path_specify = TRUE;
	break;
      }
    }

    if (path_specify) {
      dirname (strcpy (direcname, file_list + i * FILENAME_LENGTH));
      dirname (strcpy (direcname_xvp, file_list + i * FILENAME_LENGTH));
      strcat (strcat (direcname, THUMBDIR_ABS),
	      basename (file_list + i * FILENAME_LENGTH));
      strcat (strcat (direcname_xvp, THUMBDIR_ABS_XVP),
	      basename (file_list + i * FILENAME_LENGTH));
    }
    else {
      strcpy (direcname, THUMBDIR_REL);
      strcpy (direcname_xvp, THUMBDIR_REL_XVP);
      strcat (direcname, basename (file_list + i * FILENAME_LENGTH));
      strcat (direcname_xvp, basename (file_list + i * FILENAME_LENGTH));
    }

    sprintf (dname[j], "%s", file_list + i * FILENAME_LENGTH);

    if (!(status & SPECIFY_THUMB_HEIGHT) && !(status & SIZE_PROPORTIONAL)) {
      if (readXVPIC (direcname_xvp, image) || readJPEG (direcname, image)) {
	thumb_image[j] = makeXimage (d, visual, depth, image);
	free (image->data);
      }
      else if (readJPEG (dname[j], image)) {
	makeThumbnail (image, j, direcname);
      }
      else {
	if (status & VERBOSE)
	  fprintf (stderr,
		   FILE_OPEN_ERROR, argv[0], file_list + i * FILENAME_LENGTH);
	continue;
      }

    }
    else {
      if (readJPEG (dname[j], image)) {
	makeThumbnail (image, j, direcname);
      }
      else {
	if (status & VERBOSE)
	  fprintf (stderr, FILE_OPEN_ERROR,
		   argv[0], basename (file_list + i * FILENAME_LENGTH));
	continue;
      }
    }

    sprintf (fname[j], "%s", basename (file_list + i * FILENAME_LENGTH));
    if (status & DISPLAY_READING_DIALOG) {
      XFlush (d);
      rehashReadingDialog (i, j, &reading_dialog, &textarea, &gc,
			   file_list, n_file_list,
			   dialog_width, dialog_height);
    }
    j++;

    if (j > MAX_IMAGE - 1)
      break;
  }
  n_image = j;

  if (n_image == 0) {
    if (status & VERBOSE)
      fprintf (stderr, NOJPEG, argv[0]);
    exit (1);
  }
  else {
    if (status & DISPLAY_READING_DIALOG) {
      XFreeGC (d, gc);
      XDestroyWindow (d, reading_dialog);
    }
  }

  free (file_list);
  free (image);

  return n_image;
}

int
getFileList (char *file_list, int argc, char **argv, int start_index)
{
  char subdirectory = FALSE, maxfile = FALSE;
  int i, j;
  struct stat statinfo;

  for (i = start_index, j = 0; i < argc; i++) {
    subdirectory = FALSE;
    if (stat (argv[i], &statinfo))
      fprintf (stderr, NOSUCH_FILE, argv[0], argv[i]);
    if (S_ISDIR (statinfo.st_mode)) {
      j = scanDirectory (*(argv + i), statinfo,
			 &subdirectory, file_list, j, &maxfile);
    }
    else {
      sprintf (file_list + j * FILENAME_LENGTH, "%s", argv[i]);

      if (j < MAX_IMAGE - 1)
	j++;
      else
	maxfile = TRUE;
    }

    if (subdirectory) {
      if (!strcmp (argv[i], "."))
	fprintf (stderr, IGNORE_SUBS, "current directory");
      else
	fprintf (stderr, IGNORE_SUBS, argv[i]);
    }
  }

  if (maxfile)
    fprintf (stderr, TOOMANY_FILES, argv[0], MAX_IMAGE - 1);

  return j;
}

int
scanDirectory (char *directory, struct stat statinfo,
	       char *subdirectory, char *file_list, int j, char *maxfile)
{
  struct dirent **namelist;
  int iname, n;

  n = scandir (directory, &namelist, 0, alphasort);

  if (n < 0) {
    perror ("scandir");
    exit (1);
  }
  for (iname = 0; iname < n; iname++) {
    if (namelist[iname]->d_name[0] != '.') {
      char tmp[FILENAME_LENGTH];

      sprintf (tmp, "%s/%s", directory, namelist[iname]->d_name);

      stat (tmp, &statinfo);

      if (S_ISDIR (statinfo.st_mode)) {
	*subdirectory = TRUE;
	fprintf (stderr, "%s/ ", namelist[iname]->d_name);
      }
      else {
	sprintf (file_list + j * FILENAME_LENGTH, "%s", tmp);
	if (j < MAX_IMAGE - 1)
	  j++;
	else
	  *maxfile = TRUE;
      }
    }
  }
  return j;
}

void
makeThumbnail (IMG * image, int j, char *direcname)
{
  extern unsigned status;
  extern int thumb_height, size_prop_factor;
  extern XImage *thumb_image[MAX_IMAGE];
  extern int depth;
  extern Visual *visual;
  extern Display *d;

  IMG *tmp_image;

  if (status & SIZE_PROPORTIONAL)
    tmp_image = resizeImage (image, image->height / size_prop_factor);
  else
    tmp_image = resizeImage (image, thumb_height);


  thumb_image[j] = makeXimage (d, visual, depth, tmp_image);

  if (status & CREATE_INDEX) {
    unsigned int newmode;
    char target[FILENAME_LENGTH];

    newmode = 0777 & ~umask (0);

    strcpy (target, direcname);
    mkdir (dirname (direcname), newmode);

    writeJPEG (target, 75, tmp_image);
  }
  free (image->data);
  free (tmp_image->data);
  free (tmp_image);
}

void
setupReadingDialog (Window * reading_dialog, Window * textarea, GC * gc,
		    int *dialog_width, int *dialog_height)
{
  extern Display *d;

  *dialog_width = DisplayWidth (d, DefaultScreen (d)) / 3;
  *dialog_height = DisplayHeight (d, DefaultScreen (d)) - 2 * 64;

  *reading_dialog = XCreateSimpleWindow (d, RootWindow (d, 0),
					 0, 0,
					 *dialog_width,
					 *dialog_height,
					 0,
					 WhitePixel (d, 0),
					 BlackPixel (d, 0));
  *textarea = XCreateSimpleWindow (d,
				   *reading_dialog, 10, 10,
				   *dialog_width - 20, 60, 1,
				   BlackPixel (d, 0), WhitePixel (d, 0));
  XStoreName (d, *reading_dialog, "lsjpg: Reading files...");
  XMapWindow (d, *reading_dialog);
  XMapSubwindows (d, *reading_dialog);
  *gc = XCreateGC (d, *reading_dialog, 0, 0);
  XDefineCursor (d, *reading_dialog, XCreateFontCursor (d, XC_watch));
}

void
rehashReadingDialog (int i, int j, Window * reading_dialog,
		     Window * textarea, GC * gc,
		     char *file_list, int n_file_list,
		     int dialog_width, int dialog_height)
{
  extern Display *d;
  extern XImage *thumb_image[MAX_IMAGE];

  char buff[FILENAME_LENGTH];
  float x, y;

  XClearWindow (d, *textarea);
  XDrawRectangle (d, *textarea, *gc, 20, 40, dialog_width - 50, 10);

  sprintf (buff, "%s", basename (file_list + i * FILENAME_LENGTH));
  XDrawString (d, *textarea, *gc, 20, 25, buff, strlen (buff));

  sprintf (buff, "%d/%d", i + 1, n_file_list);
  XDrawString (d, *textarea, *gc, dialog_width - 90, 25, buff, strlen (buff));

  XFillRectangle (d, *textarea, *gc,
		  20, 40, (dialog_width - 50) * (i + 1) / n_file_list, 10);

  x = (dialog_width - thumb_image[j]->width) * (float) random () / RAND_MAX;
  y = (dialog_height - thumb_image[j]->height - 60)
    * (float) random () / RAND_MAX + 60;

  XPutImage (d, *reading_dialog, *gc, thumb_image[j], 0, 0, x, y,
	     thumb_image[j]->width, thumb_image[j]->height);
}

int
getFileListFromFile (char *file_list)
{
  extern unsigned status;
  char *file_list_filename = getFileListFilename ();

  char line[FILENAME_LENGTH];
  int i = 0;
  FILE *input;

  if (!(input = fopen (file_list_filename, "r"))) {
    fprintf (stderr,
	     "lsjpg: %s: No such file or directory\n", file_list_filename);
    exit (1);
  }

  while (fgets (line, FILENAME_LENGTH, input)) {
    line[strlen (line) - 1] = '\0';
    sprintf (file_list + i * FILENAME_LENGTH, "%s", line);
    i++;
  }
  fclose (input);

  if (status & VERBOSE)
    fprintf (stderr,
	     "getFileListFromFile(getimg.c): got <%d> lines in file <%s>\n",
	     i, file_list_filename);
  return i;
}
