#include "lsjpg.h"

struct my_error_mgr
{
  struct jpeg_error_mgr pub;	/* "public" fields */
  jmp_buf setjmp_buffer;	/* for return to caller */
};
typedef struct my_error_mgr *my_error_ptr;

METHODDEF (void)
my_error_exit (j_common_ptr cinfo)
{
  my_error_ptr myerr = (my_error_ptr) cinfo->err;
  /*
   *     (*cinfo->err->output_message) (cinfo);
   */
  longjmp (myerr->setjmp_buffer, 1);
}

int
readJPEG (char *filename, IMG * img)
{
  struct jpeg_decompress_struct cinfo;
  struct my_error_mgr jerr;
  FILE *infile;			/* source file */
  JSAMPARRAY buffer;		/* Output row buffer */
  int row_stride;		/* physical row width in output buffer */
  int i, cpp;

  if ((infile = fopen (filename, "rb")) == NULL) {
    /*
     * fprintf(stderr, "Can't open file: ");
     */
    return 0;
  }
  cinfo.err = jpeg_std_error (&jerr.pub);
  jerr.pub.error_exit = my_error_exit;

  if (setjmp (jerr.setjmp_buffer)) {
    jpeg_destroy_decompress (&cinfo);
    fclose (infile);
    return 0;
  }
  jpeg_create_decompress (&cinfo);

  jpeg_stdio_src (&cinfo, infile);
  (void) jpeg_read_header (&cinfo, TRUE);
  (void) jpeg_start_decompress (&cinfo);
  row_stride = cinfo.output_width * cinfo.output_components;

  buffer = (*cinfo.mem->alloc_sarray)
    ((j_common_ptr) & cinfo, JPOOL_IMAGE, row_stride, 1);

  if (cinfo.jpeg_color_space == JCS_GRAYSCALE)
    cpp = 3;
  else
    cpp = 1;

  img->data = (JSAMPLE *) malloc (cinfo.image_width * cinfo.image_height * 3);

  while (cinfo.output_scanline < cinfo.output_height) {
    (void) jpeg_read_scanlines (&cinfo, buffer, 1);
    for (i = 0; i < cpp * row_stride; i++)
      *(img->data + i + (cinfo.output_scanline - 1) * cinfo.image_width * 3) =
	*(buffer[0] + i / cpp);
  }

  img->height = cinfo.image_height;
  img->width = cinfo.image_width;
  (void) jpeg_finish_decompress (&cinfo);
  jpeg_destroy_decompress (&cinfo);
  fclose (infile);
  return 1;
}

void
writeJPEG (char *filename, int quality, IMG * img)
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  FILE *outfile;		/* target file */
  JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
  int row_stride;		/* physical row width in image buffer */
  char *image_buffer;

  image_buffer = (char *) img->data;

  cinfo.err = jpeg_std_error (&jerr);
  jpeg_create_compress (&cinfo);
  if ((outfile = fopen (filename, "wb")) == NULL) {
    fprintf (stderr, "can't open %s\n", filename);
    exit (1);
  }
  jpeg_stdio_dest (&cinfo, outfile);

  cinfo.image_width = img->width;	/* image width and height, in pixels */
  cinfo.image_height = img->height;
  cinfo.input_components = 3;	/* # of color components per pixel */
  cinfo.in_color_space = JCS_RGB;	/* colorspace of input image */

  jpeg_set_defaults (&cinfo);

  jpeg_set_quality (&cinfo, quality, TRUE);	/* limit to baseline-JPEG values */
  jpeg_start_compress (&cinfo, TRUE);
  row_stride = img->width * 3;	/* JSAMPLEs per row in image_buffer */

  while (cinfo.next_scanline < cinfo.image_height) {
    row_pointer[0] = (JSAMPROW) &image_buffer[cinfo.next_scanline * row_stride];
    (void) jpeg_write_scanlines (&cinfo, row_pointer, 1);
  }
  jpeg_finish_compress (&cinfo);
  fclose (outfile);
  jpeg_destroy_compress (&cinfo);
}

void
overwriteFile (char *flag)
{
  extern Display *d;
  extern Visual *visual;
  extern XImage *thumb_image[MAX_IMAGE];
  extern IMG *image;
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];
  extern int depth, selected_image, quality, size_prop_factor, thumb_height;
  extern unsigned status;

  IMG *tmp_image;
  char thumb[FILENAME_LENGTH], tmp[FILENAME_LENGTH];

  if (!ENABLE_OVER_WRITE) {
    fprintf (stderr,
	     "lsjpg: overwriteFile: you can *NOT* overwrite file.\nChange \"lsjpg.h\" of ENABLE_OVER_WRITE.\n");
    return;
  }

  strcpy (tmp, dname[selected_image]);
  strcpy (thumb, dirname (tmp));
  strcat (thumb, THUMBDIR_ABS);
  strcat (thumb, basename (dname[selected_image]));
  if (status & VERBOSE) {
    printf ("lsjpg: overwrite file: %s\n", thumb);
  }

  if (status & SIZE_PROPORTIONAL)
    tmp_image = resizeImage (image, image->height / size_prop_factor);
  else
    tmp_image = resizeImage (image, thumb_height);

  {
    unsigned int newmode;
    char tmp[FILENAME_LENGTH];

    newmode = 0777 & ~umask (0);
    strcpy (tmp, thumb);
    mkdir (dirname (tmp), newmode);
    writeJPEG (thumb, 75, tmp_image);
  }

  thumb_image[selected_image] = makeXimage (d, visual, depth, tmp_image);
  free (tmp_image->data);
  free (tmp_image);

  {
    extern Window thumb_window;
    extern int thumb_offset;
    extern Pixmap thumb_pmap;
    extern GC gc_big;

    XWindowAttributes xwa;

    XGetWindowAttributes (d, thumb_window, &xwa);
    thumbConfig (xwa.width);
    showSelectedRect (selected_image);
    XCopyArea (d, thumb_pmap, thumb_window, gc_big, 0,
	       -thumb_offset, xwa.width, xwa.height, 0, 0);
  }

  if (strcmp (flag, "BOTH IMAGE"))
    return;

  if (status & VERBOSE)
    printf ("lsjpg: overwrite file: %s\n", dname[selected_image]);
  writeJPEG (dname[selected_image], quality, image);
}
