#define VERSION "lsjpg 0.2.23"

#define ENABLE_OVER_WRITE                TRUE
#define JPEG_QUALITY                      100

#define THUMB_HEIGHT                       80
#define THUMB_PITCH                         5
#define SCROLLBAR_WIDTH                    12

/* #define OPAQUE_RESIZING */

#define BACKGROUND_COLOR               "black"
#define WINDOW_FRAME_BACKGROUND   "light blue"
#define BACKGROUND_BACKGROUND     "light blue"

#define FULL_SIZE_MARGIN                   35
#define TEXT_OUTPUT_WIDTH                  79
#define X_GENERATION                        5
#define ANIM_GENERATION                   256
#define ENHANCE_RATIO                     0.5
#define UNSHARP_FACTOR                   1.15
#define HALF_FACTOR                1.41421356
#define DOUBLE_FACTOR              1.41421356
#define USE_ADOMACHIC_BACKGROUND        FALSE
#define MAX_IMAGE                        2001
#define FILENAME_LENGTH                   128

#define EXIT_PUZZLE                         1
#define FAILED                              1
#define SUCCESS                             0
#define THUMBDIR_ABS      "/.lsjpg-thumbnail/"
#define THUMBDIR_REL       ".lsjpg-thumbnail/"
#define THUMBDIR_ABS_XVP           "/.xvpics/"
#define THUMBDIR_REL_XVP            ".xvpics/"
#define CONFIRMED                           1
#define PMODE_PAINT                         0
#define PMODE_LINE                          1
#define PMODE_RECTANGLE                     2
#define PMODE_CIRCLE                        3

#define USAGE_MESSAGE "usage: lsjpg [OPTION]... [FILE]...\n\
List thumbnail images of JPEG file on X (the current directory by default).\n\
Display full size image selected, and can execute some (tiny) fileters.\n\
\n\
  -b, --background-color=COLOR\t\tbackground color of thumbnail window\n\
  -d, --no-reading-dialog\t\tdon't display reading process\n\
  -f, --list-of-image=FILENAME\t\tread from image file list\n\
  -g, --geometry=GEOMETRY\t\tsize and position of thumbnail window\n\
  -h, --help\t\t\t\tdisplay this help and exit\n\
  -i, --index-create\t\t\tmake thumbnail file(.lsjpg-thumbnail/)\n\
  -k, --keep-position\t\t\tkeep fixed position of window\n\
  -l, --light-action\t\t\tlight action(no icon, pointer)\n\
  -o, --selection-output\t\toutput file name with button-2 click\n\
  -p, --pitch=WIDTH\t\t\tspecify thumbnail interval\n\
  -q, --quality=[0-100]\t\t\toverwrite JPEG with quality(default:100)\n\
  -s\t\t\t\t\tmake proportional size thumbnail\n\
      --size=FACTOR\t\t\tspecify proportional factor\n\
  -t, --thumbnail-height=HEIGHT\t\tspecify thumbnail size\n\
  -v, --version\t\t\t\toutput version info and exit\n\
  -V, --verbose\t\t\t\tfull info\n\nReport bugs to maegawa@jcom.home.ne.jp \n"

#define USAGE_MESSAGE_JAPANESE "ˡ: lsjpg [ץ]... [ե]...\n\
JPEGեΰꤵ줿᡼ɽ롣\n\
(եάϸߤΥǥ쥯ȥ JPEGեõ)\n\
\n\
  -b, --background-color=COLOR\tХå饦ɥ顼\n\
  -d, --no-reading-dialog\tɤ߹߻˥ɽʤ\n\
  -f, --list-of-image=FILENAME\t᡼Υꥹȥե뤫ɤ߹\n\
  -g, --geometry=GEOMETRY\tɽɥΥȥݥ\n\
  -h, --help\t\t\tإץåɽƽλ\n\
  -i, --index-create\t\t͡ե(ͭ)\n\
  -k, --keep-position\t\tɥɽ֤˰ˤ\n\
  -l, --light-action\t\tڤư(,ݥɽ)\n\
  -o, --selection-output\tɽɥǤܥ󥯥å\n\
\t\t\t\tǥե̾ɸϤ\n\
  -p, --pitch=WIDTH\t\tɽβδֳ֤\n\
  -q, --quality=[0-100]\t\t񤭻JPEGե(ǥե:100)\n\
  -s\t\t\t\t礭˱͡ɽ\n\
      --size=FACTOR\t\tɽΨ(1/FACTOR)\n\
  -t, --thumbnail-height=HEIGHT\tɽβι⤵\n\
  -v, --version\t\t\tɽƽλ\n\
  -V, --verbose\t\t\tĹʥå\n\
\n\
Хȯ maegawa@jcom.home.ne.jp 𤷤Ʋ\n"
/* includs */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <unistd.h>
#include <getopt.h>
#include <libgen.h>
#include <sys/stat.h>
#include <dirent.h>
#include <setjmp.h>
#include <jpeglib.h>
#include <time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/Xatom.h>

/* struct templates */
typedef struct
{
  JSAMPLE *data;		/* Points to large array of R,G,B-order data */
  int height;			/* Number of rows in image                   */
  int width;			/* Number of columns in image                */
}
IMG;

typedef struct
{
  char *data;
  int point;
}
PCELL;

/* Masks for status setting */
#define VERBOSE                  (1 << 0)
#define SIZE_PROPORTIONAL        (1 << 1)
#define CREATE_INDEX             (1 << 2)
#define SPECIFY_THUMB_HEIGHT     (1 << 3)
#define DISPLAY_READING_DIALOG   (1 << 4)
#define KEEP_POSITION            (1 << 5)
#define EXIST_BIG_WINDOW         (1 << 6)
#define EXIST_BACKGROUND         (1 << 7)
#define EXIST_CLIP               (1 << 8)
#define EXIST_PUZZLE             (1 << 9)
#define VSCROLLING               (1 << 10)
#define CONTROL_STATE            (1 << 11)
#define LIGHT_ACTION             (1 << 12)
#define PUZZLE_WITH_BORDER       (1 << 13)
#define EXIST_MENU               (1 << 14)
#define DEBUG                    (1 << 15)
#define SELECTION_OUTPUT         (1 << 16)
#define READ_FROM_FILE_LIST      (1 << 17)
#define PAINT_MODE               (1 << 18)
#define PAINT_FILL               (1 << 19)

/* FUNCTION PROTOTYPES */
int readJPEG (char *filename, IMG * img);
void writeJPEG (char *filename, int quality, IMG * img);
int initialize ();
void usage (void);
void showVersion (void);
int handleArgs (int argc, char **argv);
IMG *resizeImage (IMG * img, int height);
XImage *makeXimage (Display * d, Visual * visual, int depth, IMG * img);
int getImages (int n_opt_arg, int argc, char **argv);
void setOptionParams (void);
void swap (int *a, int *b);
void prepareThumbwindow (void);
void imageConfig (int n_image, int *pos, int width);
void handleThumbWinEvent (XEvent * event);
void handleThumbWinKey (XEvent * event);
void thumbConfig(int width);
void handleBigWinEvent (XEvent * event);
void handleBigWinKey (XEvent * event);
void handleVscroll (XEvent * event);
int handleCommonKey (XEvent * event);
void displaySelectedImage (void);
void killBigWindow (void);
void vscrollConfig (void);
void selectFilter (char *buf, IMG * image);
void fullSize (void);
void setSuitableWinPlace (void);
void makeClipRect (int x1, int x2, int y1, int y2);
void makeClipCircle (int x1, int x2, int y1, int y2);
IMG *cropImage (IMG * image);
void unSharpMask (IMG * image);
void setupReadingDialog (Window * reading_dialog, Window * textarea,
			 GC * gc, int *w, int *h);
void rehashReadingDialog (int i, int j, Window * reading_dialog,
			  Window * textarea, GC * gc,
			  char *flist, int n_flist, int w, int h);
void median (IMG * image);
void xyConv (char * xy, IMG * image);
void zoom  (IMG * image);
int cellMdn  (int a, int b, int c, int d, int e, int f, int g, int h, int i);
int xCellAtm (int a, int b, int c, int d, int e, int f, int g, int h, int i);
int yCellAtm (int a, int b, int c, int d, int e, int f, int g, int h, int i);
void XZanim (char * XZ, IMG * image);
int XCellAtm (int a, int b, int c, int d, int e, int f, int g, int h, int i);
int ZCellAtm (int a, int b, int c, int d, int e, int f, int g, int h, int i);
void grayScale (IMG * image);
void reverseColor    (IMG * image);
void blackWhite      (IMG * image);
void edgeEnhance     (IMG * image);
void clockwiseRotate (IMG * image);
void lightUp         (IMG * image);
void keen            (IMG * image);
void ifunction       (IMG * image);
void jfunction       (IMG * image);
void nfunction       (IMG * image);
void opposite        (IMG * image);
void windowFrame     (IMG * image);
void thinner         (IMG * image);
void vibrate         (IMG * image);
void edgeDetect      (IMG * image);
void textOutput      (IMG * image);
void setToRootWindow (IMG * image);
void addToRootWindow (IMG * image);
void resizeBigWindow (int h);
void changeWindowSize (int w, int h);
void windowConfig (void);
void markSelectedImage (int selected_image, int previous_image);
void adjustImageToWindowSize (void);
void bigWindowGoHome (void);
int seekSelectedImage (int x, int y);
int scanDirectory (char *directory, struct stat statinfo, char *subdirectory, char *flist, int j, char *maxfile);
void makeThumbnail(IMG *image, int j, char *direcname);
void thumbWinControlKey(int ks, XWindowAttributes *xwa);
int getFileList(char *file_list, int argc, char **argv, int start_index);
int getFileListFromFile(char *file_list);
void makeIcon(void);
void rehashThumbWin(XWindowAttributes *xwa);
void YFunction (IMG * image);
void puzzle (IMG * image);
int * getRandomSequence(int n);
void handlePuzzleEvent(XEvent * event);
void rehashPuzzleState(void);
int getPuzzlePoint(int x, int y);
void checkPuzzleState(void);
void freePcell(void);
int handlePuzzleKey(KeySym ks);
int handleControlState(KeySym ks);
int handleNonControlState(KeySym ks);
void downLineCell(void);
void upLineCell(void);
void leftLineCell(void);
void rightLineCell(void);
void solvePuzzleOneStroke(void);
void horizontalForwardCell(void);
void horizontalBackwardCell(void);
void verticalForwardCell(void);
void verticalBackwardCell(void);
void puzzleHelp(void);
void puzzleHelpReturn(void);
void puzzleComplete(void);
void puzzleScoreDisplay(void);
void pmenuSetup(void);
void menuSetup(XEvent * event);
void pmenuSetupItem(void);
void menuSetupItem(void);
void handlePmenuWindow(XEvent * event);
void handleMenuWindow(XEvent * event);
void handlePmenuButtonPress(XEvent * event);
void handleMenuButtonPress(XEvent * event);
void disposeMenu(void);
void disposePmenu(void);
void filterInterface(char * kbuff);
void handleControlKey(XEvent * event);
void fillImageToRootWindow(void);
void stdoutReceivedEvent(XEvent * event);
void handleAutoScroll(XWindowAttributes * xwa);
void handleMenuPointerMotion(XEvent * event);
char * getThumbwinColor(void);
char * getThumbwinGeometry(void);
char * getFileListFilename(void);
void showSelectedRect(int select);
void overwriteFile(char * flag);
void deleteCurrentImageFile(void);
void deleteImageFromList(int select);
int confirmDialog(char * message);
void createThumbPixmap(int w, int h);
void scrollDown(void);
void scrollUp(void);
void initializePaintGC(void);
void allocColors1(void);
void allocColors2(void);
Pixmap setTilePattern (void);
void pmenuShowCurrentSelection(void);
void changeStippleMask(int button);
int readXVPIC(char *filepath, IMG *image);
