#include "lsjpg.h"
#define MWIDTH               115
#define MHEIGHT              525
#define MPWIDTH              115
#define MPHEIGHT             575
#define ITEM_HEIGHT           20
#define POINTER_COLOR       "red"
#define BUTTON_COLOR     "gray80"
#define BACKGROUND       "gray85"

extern Display *d;
extern Window menu_window, pmenu_window;

static char color1_allocated = FALSE;
static char color2_allocated = FALSE;
static char gc_initialized = FALSE;

static GC gc_menu, gc_pmenu;
static Pixmap menu_pmap, pmenu_pmap;
static Pixmap mask1, mask2, mask3, mask_lsjpg;

static Colormap default_cmap;
static XColor pointer_color, button_color, bgcolor, red, green, blue, yellow, magenta, cyan, gray;
static XGCValues gcv;

static char *menu_item[] = {
  "NEXT ->", "  <- PREVIOUS",
  "add to Root", "black/White", "clockwise", "double", "edge Enhance",
  "full Screen", "gray Scale", "half Size", "impressed", "juicy", "keen",
  "light Up", "median", "neutral", "opposite", "puzzle", "quit", "reverse",
  "set to Root", "thin", "unsharp", "vibrate", "window frame", "PAINT =>"
};

void
initializePaintGC (void)
{
  extern Display *d;
  extern Window big_window;

#include "lsjpg.xbm"
  static char mask1_bits[] = {
    0x11, 0x00, 0x44, 0x00, 0x11, 0x00, 0x44, 0x00 };
  static char mask2_bits[] = {
    0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa };
  static char mask3_bits[] = {
    0xee, 0xff, 0xbb, 0xff, 0xee, 0xff, 0xbb, 0xff };

  gcv.line_width = 9;
  gcv.foreground = red.pixel;
  gcv.background = BlackPixel (d, 0);
  gcv.line_style = LineSolid;
  gcv.cap_style = CapRound;
  gcv.join_style = JoinRound;
  gcv.fill_style = FillSolid;
  gcv.fill_rule = EvenOddRule;
    
  mask1 = XCreatePixmapFromBitmapData (d, big_window, mask1_bits,
				       8, 8, 1L, 0L, 1);
  mask2 = XCreatePixmapFromBitmapData (d, big_window, mask2_bits,
				       8, 8, 1L, 0L, 1);
  mask3 = XCreatePixmapFromBitmapData (d, big_window, mask3_bits,
				       8, 8, 1L, 0L, 1);
  mask_lsjpg = XCreatePixmapFromBitmapData (d, big_window, lsjpg_bits,
					    lsjpg_width, lsjpg_height,
					    1L, 0L, 1);
  gcv.stipple = mask2;

  if(gcv.tile)
    XFreePixmap(d, gcv.tile);
  gcv.tile = setTilePattern();

  gc_initialized = TRUE;
}

Pixmap
setTilePattern (void)
{
  extern Display *d;
  extern Window big_window;
  extern int depth;

  Pixmap tile;
  GC gc;
  int i, j;

  tile = XCreatePixmap(d, big_window, 128, 128, depth);
  gc = XCreateGC (d, big_window, 0, 0);

  for (i = 0; i < 128; i++) {
    for (j = 0; j < 128; j++) {
      XSetForeground (d, gc, random());
      XDrawPoint(d, tile, gc, i, j);
    }
  }

  XFreeGC(d, gc);
  return tile;
}

void
allocColors1 (void)
{
  default_cmap = DefaultColormap (d, DefaultScreen (d));
  XParseColor (d, default_cmap, POINTER_COLOR, &pointer_color);
  XParseColor (d, default_cmap, BUTTON_COLOR, &button_color);
  XParseColor (d, default_cmap, BACKGROUND, &bgcolor);
  XAllocColor (d, default_cmap, &pointer_color);
  XAllocColor (d, default_cmap, &button_color);
  XAllocColor (d, default_cmap, &bgcolor);

  color1_allocated = TRUE;
}

void
allocColors2 (void)
{
  XParseColor (d, default_cmap, "red", &red);
  XParseColor (d, default_cmap, "green", &green);
  XParseColor (d, default_cmap, "blue", &blue);
  XParseColor (d, default_cmap, "yellow", &yellow);
  XParseColor (d, default_cmap, "magenta", &magenta);
  XParseColor (d, default_cmap, "cyan", &cyan);
  XParseColor (d, default_cmap, "gray", &gray);
  XAllocColor (d, default_cmap, &red);
  XAllocColor (d, default_cmap, &green);
  XAllocColor (d, default_cmap, &blue);
  XAllocColor (d, default_cmap, &yellow);
  XAllocColor (d, default_cmap, &magenta);
  XAllocColor (d, default_cmap, &cyan);
  XAllocColor (d, default_cmap, &gray);

  color2_allocated = TRUE;
}

void
handleMenuWindow (XEvent * event)
{
  switch (event->type) {
  case Expose:
    XCopyArea (d, menu_pmap, menu_window, gc_menu,
	       0, 0, MWIDTH, MHEIGHT, 0, 0);
    break;

  case ButtonPress:
    if (event->xbutton.button == Button1)
      handleMenuButtonPress (event);
    break;

  case MotionNotify:
    handleMenuPointerMotion (event);
    XCopyArea (d, menu_pmap, menu_window, gc_menu,
	       0, 0, MWIDTH, MHEIGHT, 0, 0);
    break;

  default:
    ;
  }
}

void
handleMenuButtonPress (XEvent * event)
{
  extern IMG *image;

  int item;

  for (item = 0; item < sizeof (menu_item) / sizeof (char *); item++) {
    if (event->xbutton.y < (item + 1) * 20)
      break;
  }

  switch (item) {
  case 0:
    handleControlState (XK_n);
    break;
  case 1:
    handleControlState (XK_p);
    break;
  case 2:
    addToRootWindow (image);
    killBigWindow ();
    break;
  case 3:
    filterInterface ("b");
    break;
  case 4:
    selectFilter ("c", image);
    break;
  case 5:
    filterInterface ("d");
    break;
  case 6:
    filterInterface ("e");
    break;
  case 7:
    selectFilter ("f", image);
    break;
  case 8:
    filterInterface ("g");
    break;
  case 9:
    selectFilter ("h", image);
    break;
  case 10:
    filterInterface ("i");
    break;
  case 11:
    filterInterface ("j");
    break;
  case 12:
    filterInterface ("k");
    break;
  case 13:
    filterInterface ("l");
    break;
  case 14:
    filterInterface ("m");
    break;
  case 15:
    filterInterface ("n");
    break;
  case 16:
    filterInterface ("o");
    break;
  case 17:
    selectFilter ("p", image);
    disposeMenu ();
    break;
  case 18:
    killBigWindow ();
    break;
  case 19:
    filterInterface ("r");
    break;
  case 20:
    setToRootWindow (image);
    killBigWindow ();
    break;
  case 21:
    filterInterface ("t");
    break;
  case 22:
    filterInterface ("u");
    break;
  case 23:
    filterInterface ("v");
    break;
  case 24:
    filterInterface ("w");
    break;
  case 25:
    disposeMenu();
    pmenuSetup();
    break;
  default:
    ;
  }
}

void
disposeMenu (void)
{
  extern unsigned status;

  XFreeGC (d, gc_menu);
  XFreePixmap (d, menu_pmap);
  XDestroyWindow (d, menu_window);

  status &= ~EXIST_MENU;
}

void
handleMenuPointerMotion (XEvent * event)
{
  int i;

  menuSetupItem ();
  XSetForeground (d, gc_menu, pointer_color.pixel);
  for (i = 0; i < sizeof (menu_item) / sizeof (char *); i++) {
    if (event->xbutton.y < (i + 1) * ITEM_HEIGHT) {
      XDrawString (d, menu_pmap, gc_menu,
		   12, (i + 1) * ITEM_HEIGHT - 3,
		   menu_item[i], strlen (menu_item[i]));
      return;
    }
  }
}

void
menuSetupItem (void)
{
  int i;

  XSetForeground (d, gc_menu, bgcolor.pixel);
  XFillRectangle (d, menu_pmap, gc_menu, 0, 0, MWIDTH, MHEIGHT);

  /* psudo 3-D effect */
  XSetForeground (d, gc_menu, WhitePixel (d, 0));
  XDrawRectangle (d, menu_pmap, gc_menu, 0, 0, MWIDTH, MHEIGHT);
  XDrawRectangle (d, menu_pmap, gc_menu, 1, 1, MWIDTH, MHEIGHT);
  XSetForeground (d, gc_menu, BlackPixel (d, 0));
  XDrawRectangle (d, menu_pmap, gc_menu, -1, -1, MWIDTH, MHEIGHT);

  XSetForeground (d, gc_menu, button_color.pixel);
  for (i = 1; i < 24; i++) {
    XFillRectangle (d, menu_pmap, gc_menu, 10, 20 * i + 21, MWIDTH - 20, 18);
  }

  XSetForeground (d, gc_menu, BlackPixel (d, 0));

  for (i = 0; i < sizeof (menu_item) / sizeof (char *); i++) {
    XDrawString (d, menu_pmap, gc_menu,
		 12, (i + 1) * 20 - 3, menu_item[i], strlen (menu_item[i]));
  }
}

void
menuSetup (XEvent * event)
{
  extern unsigned status;
  extern int depth;
  extern Window big_window;

  Window child;
  XWindowAttributes xwa;
  XSetWindowAttributes attr;
  int x0, y0;

  status |= EXIST_MENU;

  if (color1_allocated == FALSE)
    allocColors1 ();

  XGetWindowAttributes (d, big_window, &xwa);
  XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			 0, 0, &xwa.x, &xwa.y, &child);

  x0 = xwa.x + event->xbutton.x;
  y0 = xwa.y + event->xbutton.y;
  if (DisplayWidth (d, DefaultScreen (d)) < x0 + MWIDTH)
    x0 = DisplayWidth (d, DefaultScreen (d)) - MWIDTH;
  if (DisplayHeight (d, DefaultScreen (d)) < y0 + MHEIGHT)
    y0 = DisplayHeight (d, DefaultScreen (d)) - MHEIGHT;


  menu_window = XCreateSimpleWindow (d, RootWindow (d, 0),
				     x0, y0, MWIDTH, MHEIGHT, 0,
				     BlackPixel (d, 0), bgcolor.pixel);
  gc_menu = XCreateGC (d, menu_window, 0, 0);
  XDefineCursor (d, menu_window, XCreateFontCursor (d, XC_right_ptr));

  XSelectInput (d, menu_window,
		ExposureMask | ButtonPressMask |
		KeyPressMask | KeyReleaseMask | PointerMotionMask);

  attr.override_redirect = True;

  XChangeWindowAttributes (d, menu_window, CWOverrideRedirect, &attr);
  XMapWindow (d, menu_window);

  menu_pmap = XCreatePixmap (d, menu_window, MWIDTH, MHEIGHT, depth);

  menuSetupItem ();

  XFlush (d);
}

int
confirmDialog (char *message)
{
  extern unsigned status;
  extern Display *d;
  extern Window big_window;
  GC gc;
  Window dialog;
  XEvent event;
  char buff[FILENAME_LENGTH];
  int x, y;

  if (color1_allocated == FALSE) {
    allocColors1 ();
    color1_allocated = TRUE;
  }

  dialog = XCreateSimpleWindow (d, big_window, 10, 10, 120, 47, 0,
				BlackPixel (d, 0), bgcolor.pixel);
  XMapSubwindows (d, big_window);
  gc = XCreateGC (d, dialog, 0, 0);
  XDefineCursor (d, dialog, XCreateFontCursor (d, XC_top_left_arrow));
  XSelectInput (d, dialog,
		ButtonPressMask | KeyPressMask | PointerMotionMask);

  /* psudo 3-D effect */
  XSetForeground (d, gc, WhitePixel (d, 0));
  XDrawRectangle (d, dialog, gc, 0, 0, 120, 47);
  XDrawRectangle (d, dialog, gc, 1, 1, 120, 47);
  XSetForeground (d, gc, BlackPixel (d, 0));
  XDrawRectangle (d, dialog, gc, -1, -1, 120, 47);

  strcpy (buff, message);
  strcat (buff, " file?");
  XDrawString (d, dialog, gc, 5, 15, buff, strlen (buff));
  XDrawString (d, dialog, gc, 15, 35, "Yes", 3);
  XDrawRectangle (d, dialog, gc, 52, 20, 30, 20);
  XDrawString (d, dialog, gc, 60, 35, "No", 2);
  XFlush (d);

  while (1) {
    XNextEvent (d, &event);
    /*    if (status & DEBUG)
       stdoutReceivedEvent (&event); */

    if (event.type == MotionNotify) {
      XSetForeground (d, gc, BlackPixel (d, 0));
      XDrawString (d, dialog, gc, 5, 15, buff, strlen (buff));

      if (10 < event.xbutton.x && event.xbutton.x < 40 &&
	  20 < event.xbutton.y && event.xbutton.y < 40) {
	XSetForeground (d, gc, BlackPixel (d, 0));
	XDrawRectangle (d, dialog, gc, 10, 20, 30, 20);
	XSetForeground (d, gc, WhitePixel (d, 0));
	XFillRectangle (d, dialog, gc, 10, 20, 30, 20);
	XSetForeground (d, gc, pointer_color.pixel);
	XDrawString (d, dialog, gc, 15, 35, "Yes", 3);
      }
      else {
	XSetForeground (d, gc, bgcolor.pixel);
	XFillRectangle (d, dialog, gc, 10, 20, 31, 21);
	XSetForeground (d, gc, BlackPixel (d, 0));
	XDrawString (d, dialog, gc, 15, 35, "Yes", 3);
      }

      if (52 < event.xbutton.x && event.xbutton.x < 82 &&
	  20 < event.xbutton.y && event.xbutton.y < 40) {
	XSetForeground (d, gc, WhitePixel (d, 0));
	XFillRectangle (d, dialog, gc, 52, 20, 30, 20);
	XSetForeground (d, gc, pointer_color.pixel);
	XDrawString (d, dialog, gc, 60, 35, "No", 2);
      }
      else {
	XSetForeground (d, gc, bgcolor.pixel);
	XFillRectangle (d, dialog, gc, 52, 20, 30, 20);
	XSetForeground (d, gc, BlackPixel (d, 0));
	XDrawRectangle (d, dialog, gc, 52, 20, 30, 20);
	XDrawString (d, dialog, gc, 60, 35, "No", 2);
      }
    }

    if (event.type == ButtonPress || event.type == KeyPress)
      break;
  }

  XFreeGC (d, gc);
  XDestroyWindow (d, dialog);

  if (event.type == KeyPress) {
    char kbuff[16];
    KeySym ks;
    XLookupString ((XKeyEvent *) & event, kbuff, sizeof (kbuff),
		   &ks, (XComposeStatus *) NULL);
    if (ks == XK_y) {
      return CONFIRMED;
    }
  }

  if (event.xany.window == dialog && event.type == ButtonPress) {
    x = event.xbutton.x;
    y = event.xbutton.y;
    if (10 < x && x < 40 && 20 < y && y < 40)
      return CONFIRMED;
  }
  status &= ~CONTROL_STATE;
  return 0;
}

void
disposePmenu (void)
{
  extern unsigned status;
  extern Window big_window;

  XFreeGC (d, gc_menu);
  XFreePixmap (d, pmenu_pmap);
  XDestroyWindow (d, pmenu_window);
  status &= ~PAINT_MODE;
  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));
}

void
pmenuSetup (void)
{
  extern unsigned status;
  extern int depth;
  extern Window big_window;

  Window child;
  XWindowAttributes xwa;
  XSetWindowAttributes attr;
  int x0, y0;

  status |= PAINT_MODE;

  if (color2_allocated == FALSE)
    allocColors2 ();
  if (gc_initialized == FALSE)
    initializePaintGC();

  XGetWindowAttributes (d, big_window, &xwa);
  XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			 0, 0, &xwa.x, &xwa.y, &child);
  if (xwa.x < DisplayWidth (d, DefaultScreen (d)) - xwa.x - xwa.width)
    x0 = xwa.x + xwa.width;
  else
    x0 = xwa.x - MWIDTH;
  y0 = xwa.y - 21;

  pmenu_window = XCreateSimpleWindow (d, RootWindow (d, 0),
				      x0, y0, MPWIDTH, MPHEIGHT, 0,
				      BlackPixel (d, 0), bgcolor.pixel);
  gc_pmenu = XCreateGC (d, pmenu_window, 0, 0);
  XDefineCursor (d, pmenu_window, XCreateFontCursor (d, XC_right_ptr));
  XSelectInput (d, pmenu_window,
		ExposureMask | ButtonPressMask |
		KeyPressMask | KeyReleaseMask | PointerMotionMask);
  attr.override_redirect = True;
  XChangeWindowAttributes (d, pmenu_window, CWOverrideRedirect, &attr);
  XMapWindow (d, pmenu_window);
  pmenu_pmap = XCreatePixmap (d, pmenu_window, MPWIDTH, MPHEIGHT, depth);
  pmenuSetupItem ();

  XDefineCursor (d, big_window, XCreateFontCursor (d, XC_pencil));
}

void
pmenuSetupItem (void)
{
  extern GC gc_paint;

  XChangeGC(d, gc_paint, GCForeground | GCBackground |
	    GCLineWidth | GCLineStyle | GCCapStyle | GCJoinStyle |
	    GCFillStyle | GCStipple | GCTile, &gcv);

  XSetForeground (d, gc_pmenu, bgcolor.pixel);
  XSetLineAttributes( d, gc_pmenu, 1, LineSolid, CapRound, JoinRound );
  XFillRectangle (d, pmenu_pmap, gc_pmenu, 0, 0, MPWIDTH, MPHEIGHT);

  /* psudo 3-D effect */
  XSetForeground (d, gc_pmenu, WhitePixel (d, 0));
  XDrawRectangle (d, pmenu_pmap, gc_pmenu, 0, 0, MPWIDTH, MPHEIGHT);
  XDrawRectangle (d, pmenu_pmap, gc_pmenu, 1, 1, MPWIDTH, MPHEIGHT);
  XSetForeground (d, gc_pmenu, BlackPixel (d, 0));
  XDrawRectangle (d, pmenu_pmap, gc_pmenu, -1, -1, MPWIDTH, MPHEIGHT);

  XSetForeground (d, gc_pmenu, BlackPixel (d, 0));
  XSetLineAttributes( d, gc_pmenu, 9, LineSolid, CapRound, JoinRound );
  XDrawLine(d, pmenu_pmap, gc_pmenu, 8, 25, 8, 25);
  XSetLineAttributes( d, gc_pmenu, 18, LineSolid, CapRound, JoinRound );
  XDrawLine(d, pmenu_pmap, gc_pmenu, 25, 25, 25, 25);
  XSetLineAttributes( d, gc_pmenu, 27, LineSolid, CapRound, JoinRound );
  XDrawLine(d, pmenu_pmap, gc_pmenu, 52, 25, 52, 25);
  XSetLineAttributes( d, gc_pmenu, 36, LineSolid, CapRound, JoinRound );
  XDrawLine(d, pmenu_pmap, gc_pmenu, 88, 25, 88, 25);

  XSetForeground (d, gc_pmenu, BlackPixel (d, 0));
  XSetLineAttributes( d, gc_pmenu, 1, LineSolid, CapRound, JoinRound );

  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 65, "LineSolid", 9);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 85, "OnOffDash", 9);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 105, "DoubleDash", 10);

  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 130, "CapRound", 8);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 150, "CapNotLast", 10);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 170, "CapButt", 7);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 190, "Projecting", 10);

  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 215, "JoinRonud", 9);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 235, "JoinBevel", 9);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 255, "JoinMiter", 9);

  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 280, "FillSolid", 9);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 300, "FillTiled", 9);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 320, "FillStippled", 12);
  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 340, "OpaqueStippled", 14);

  XDrawString(d, pmenu_pmap, gc_pmenu, 10, 365, "EvnOdd", 6);
  XDrawString(d, pmenu_pmap, gc_pmenu, 60, 365, "Windng", 6);

  XDrawString(d, pmenu_pmap, gc_pmenu, 13, 390, "Paint", 5);
  XDrawString(d, pmenu_pmap, gc_pmenu, 60, 390, "Draw", 4);
  XDrawString(d, pmenu_pmap, gc_pmenu, 13, 410, "Rect", 4);
  XDrawString(d, pmenu_pmap, gc_pmenu, 60, 410, "Circle", 6);

  XDrawString(d, pmenu_pmap, gc_pmenu, 65, 435, "Fill", 4);

  XSetForeground (d, gc_pmenu, red.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 8, 450, 20, 20);
  XSetForeground (d, gc_pmenu, green.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 33, 450, 20, 20);
  XSetForeground (d, gc_pmenu, blue.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 58, 450, 20, 20);
  XSetForeground (d, gc_pmenu, yellow.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 83, 450, 20, 20);

  XSetForeground (d, gc_pmenu, magenta.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 8, 475, 20, 20);
  XSetForeground (d, gc_pmenu, cyan.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 33, 475, 20, 20);
  XSetForeground (d, gc_pmenu, gray.pixel);
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 58, 475, 20, 20);
  XSetForeground (d, gc_pmenu, BlackPixel (d, 0));
  XFillRectangle(d, pmenu_pmap, gc_pmenu, 83, 475, 20, 20);

  pmenuShowCurrentSelection();
}

void
pmenuShowCurrentSelection(void)
{
  extern unsigned status, paint_mode_type;
  extern GC gc_paint;

  switch (gcv.line_width) {
  case 9: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 3, 20, 10, 10);
    break;
  case 18: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 15, 15, 20, 20);
    break;
  case 27: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 37, 10, 30, 30);
    break;
  case 36: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 68, 5, 40, 40);
    break;
  default:
    ;
  }

  switch (gcv.line_style) {
  case LineSolid: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 50, 98, 18);
    break;
  case LineOnOffDash: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 70, 98, 18);
    break;
  case LineDoubleDash: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 90, 98, 18);
    break;
  default:
    ;
  }
  switch (gcv.cap_style) {
  case CapRound: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 115, 98, 18);
    break;
  case CapNotLast: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 135, 98, 18);
    break;
  case CapButt: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 155, 98, 18);
    break;
  case CapProjecting: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 175, 98, 18);
    break;
  default:
    ;
  }

  switch (gcv.join_style) {
  case JoinRound: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 200, 98, 18);
    break;
  case JoinBevel: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 220, 98, 18);
    break;
  case JoinMiter: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 5, 240, 98, 18);
    break;
  default:
    ;
  }

  switch (gcv.fill_style) {
  case FillSolid: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 265, 98, 18);
    break;
  case FillTiled: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 285, 98, 18);
    break;
  case FillStippled: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 5, 305, 98, 18);
    break;
  case FillOpaqueStippled: 
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 5, 325, 98, 18);
    break;
  default:
    ;
  }

  switch (gcv.fill_rule) {
  case EvenOddRule:
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 350, 48, 18);
    break;
  case WindingRule:
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 57, 350, 48, 18);
    break;
  default:
    ;
  }

  switch (paint_mode_type) {
  case PMODE_PAINT:
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 375, 48, 18);
    break;
  case PMODE_LINE:
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 57, 375, 48, 18);
    break;
  case PMODE_RECTANGLE:
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 6, 395, 48, 18);
    break;
  case PMODE_CIRCLE:
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 57, 395, 48, 18);
    break;

  default:
    ;
  }

  if (status & PAINT_FILL)
    XDrawRectangle(d, pmenu_pmap, gc_pmenu, 60, 420, 40, 17);

  switch (paint_mode_type) {
  case PMODE_PAINT:
  case PMODE_LINE:
    XDrawLine(d, pmenu_pmap, gc_paint, 17, 515, 57, 515);
    XDrawLine(d, pmenu_pmap, gc_paint, 57, 515, 95, 555);
    break;
  case PMODE_RECTANGLE:
    if (status & PAINT_FILL)
      XFillRectangle(d, pmenu_pmap, gc_paint, 17, 515, 78, 40);
    else
      XDrawRectangle(d, pmenu_pmap, gc_paint, 17, 515, 78, 40);
    break;
  case PMODE_CIRCLE:
    if (status & PAINT_FILL)
      XFillArc(d, pmenu_pmap, gc_paint, 17, 515, 78, 40, 0, 360*64);
    else
      XDrawArc(d, pmenu_pmap, gc_paint, 17, 515, 78, 40, 0, 360*64);
    break;

  default:
    ;
  }
}

void
handlePmenuWindow (XEvent * event)
{
  extern Window big_window;

  switch (event->type) {
  case Expose:
    XCopyArea (d, pmenu_pmap, pmenu_window, gc_pmenu,
	       0, 0, MPWIDTH, MPHEIGHT, 0, 0);
    break;

  case ButtonPress:
    XMapRaised (d, big_window);
    handlePmenuButtonPress (event);
    XCopyArea (d, pmenu_pmap, pmenu_window, gc_pmenu,
	       0, 0, MPWIDTH, MPHEIGHT, 0, 0);
    break;

  default:
    ;
  }
}

void
handlePmenuButtonPress (XEvent * event)
{
  extern unsigned status, paint_mode_type;

  if (event->xbutton.y < 45) {
    if (event->xbutton.x < 13)
      gcv.line_width = 9;
    else if (event->xbutton.x < 35)
      gcv.line_width = 18;
    else if (event->xbutton.x < 67)
      gcv.line_width = 27;
    else
      gcv.line_width = 36;
  }

  else if (event->xbutton.y < 75) {
    gcv.cap_style = CapRound;
    gcv.line_style = LineSolid;
  }
  else if (event->xbutton.y < 95) {
    gcv.cap_style = CapNotLast;
    gcv.line_style = LineOnOffDash;
  }
  else if (event->xbutton.y < 115)
    gcv.line_style = LineDoubleDash;

  else if (event->xbutton.y < 140)
    gcv.cap_style = CapRound;
  else if (event->xbutton.y < 160)
    gcv.cap_style = CapNotLast;
  else if (event->xbutton.y < 180)
    gcv.cap_style = CapButt;
  else if (event->xbutton.y < 200)
    gcv.cap_style = CapProjecting;

  else if (event->xbutton.y < 225)
    gcv.join_style = JoinRound;
  else if (event->xbutton.y < 245)
    gcv.join_style = JoinBevel;
  else if (event->xbutton.y < 265)
    gcv.join_style = JoinMiter;

  else if (event->xbutton.y < 290)
    gcv.fill_style = FillSolid;
  else if (event->xbutton.y < 310)
    gcv.fill_style = FillTiled;
  else if (event->xbutton.y < 330)
    gcv.fill_style = FillStippled;
  else if (event->xbutton.y < 350)
    gcv.fill_style = FillOpaqueStippled;

  else if (event->xbutton.y < 375)
    if (event->xbutton.x < 51)
      gcv.fill_rule = EvenOddRule;
    else 
      gcv.fill_rule = WindingRule;

  else if (event->xbutton.y < 400) {
    if (event->xbutton.x < 51) {
      paint_mode_type = PMODE_PAINT;
      status &= ~PAINT_FILL;
    }
    else {
      paint_mode_type = PMODE_LINE;
      status &= ~PAINT_FILL;
    }
  }
  else if (event->xbutton.y < 420) {
    if (event->xbutton.x < 51)
      paint_mode_type = PMODE_RECTANGLE;
    else
      paint_mode_type = PMODE_CIRCLE;
  }

  else if (event->xbutton.y < 445) {
    if (paint_mode_type == PMODE_RECTANGLE ||
	paint_mode_type == PMODE_CIRCLE) {
      if (status & PAINT_FILL) status &= ~PAINT_FILL;
      else status |= PAINT_FILL;
    }
  }

  else if (event->xbutton.y < 470) {
    if (event->xbutton.x < 28) {
      if (event->xbutton.button == Button1)
	gcv.foreground = red.pixel;
      else
	gcv.background = red.pixel;
    }
    else if (event->xbutton.x < 53) {
      if (event->xbutton.button == Button1)
	gcv.foreground = green.pixel;
      else
	gcv.background = green.pixel;
    }
    else if (event->xbutton.x < 78) {
      if (event->xbutton.button == Button1)
	gcv.foreground = blue.pixel;
      else
	gcv.background = blue.pixel;
    }
    else {
      if (event->xbutton.button == Button1)
	gcv.foreground = yellow.pixel;
      else
	gcv.background = yellow.pixel;
    }
  }
  else if (event->xbutton.y < 495) {
    if (event->xbutton.x < 28) {
      if (event->xbutton.button == Button1)
	gcv.foreground = magenta.pixel;
      else
	gcv.background = magenta.pixel;
    }
    else if (event->xbutton.x < 53) {
      if (event->xbutton.button == Button1)
	gcv.foreground = cyan.pixel;
      else
	gcv.background = cyan.pixel;
    }
    else if (event->xbutton.x < 78) {
      if (event->xbutton.button == Button1)
	gcv.foreground = gray.pixel;
      else
	gcv.background = gray.pixel;
    }
    else {
      if (event->xbutton.button == Button1)
	gcv.foreground = BlackPixel (d, 0);
      else
	gcv.background = BlackPixel (d, 0);
    }
  }
  else {
    if (gcv.fill_style == FillStippled || gcv.fill_style == FillOpaqueStippled)
      changeStippleMask(event->xbutton.button);
  }
  
  pmenuSetupItem ();
}

void
changeStippleMask(int button)
{
  static char current_mask = 1;

  if (button == 1)
    current_mask++;
  else
    current_mask--;

  if (current_mask < 0) current_mask = 3;
  if (3 < current_mask) current_mask = 0;

  switch (current_mask) {
  case 0:
    gcv.stipple = mask1;
    break;
  case 1:
    gcv.stipple = mask2;
    break;
  case 2:
    gcv.stipple = mask3;
    break;
  case 3:
    gcv.stipple = mask_lsjpg;
    break;
  default:
    ;
  }
}
