#include "lsjpg.h"
#define PUZZLE_HELP "PUZZLE HELP: invalid cell %d\n"

extern unsigned status;
static PCELL *pcell;
static Pixmap small_pmap;
static char exist_small_pmap = FALSE;
static int npcell, npx, npy, xm, ym;
static int cell_size;
static int score;
static int *tmp_sequence;
static int *hint_sequence;

void
puzzle (IMG * image)
{
  extern int key_buffer;

  int ndiv;
  int i, j, k, ij;
  int ip, jp, ijp;
  int *rs;

  status |= EXIST_PUZZLE;

  if (2 < key_buffer)
    ndiv = key_buffer;
  else
    ndiv = 5;

  if (image->height < image->width)
    cell_size = image->width / ndiv;
  else
    cell_size = image->height / ndiv;

  npx = image->width / cell_size;

  while (npx <= 1) {
    cell_size--;
    npx = image->width / cell_size;
  }
  npy = image->height / cell_size;
  while (npy <= 1) {
    cell_size--;
    npy = image->height / cell_size;
  }
  npx = image->width / cell_size;
  xm = image->width % cell_size / 2;
  ym = image->height % cell_size / 2;
  npcell = npx * npy;
  score = 4 * npcell;

  pcell = (PCELL *) malloc (sizeof (PCELL) * npcell);
  for (i = 0; i < npcell; i++) {
    (pcell + i)->data = malloc (cell_size * cell_size * 3);
  }

  rs = getRandomSequence (npcell);

  for (ijp = 0; ijp < npcell; ijp++) {
    ip = ijp / npx;
    jp = ijp % npx;
    (pcell + ijp)->point = *(rs + ijp);

    for (ij = 0; ij < cell_size * cell_size; ij++) {
      i = ij / cell_size;
      j = ij % cell_size;
      for (k = 0; k < 3; k++) {
	*((pcell + jp + ip * npx)->data + 3 * ij + k) =
	  image->data[xm * 3 + ym * image->width * 3 + k +
		      j * 3 + i * image->width * 3 +
		      jp * cell_size * 3 + ip * image->width * cell_size * 3];
      }
    }
  }

  rehashPuzzleState ();
  puzzleScoreDisplay ();

  free (rs);
  tmp_sequence = (int *) malloc (sizeof (int) * npcell);
}

void
rehashPuzzleState (void)
{
  extern int depth;
  extern IMG *image;
  extern XImage *big_image;
  extern Pixmap big_pmap;
  extern Window big_window;
  extern GC gc_big;
  extern Visual *visual;
  extern Display *d;

  int ijp, ip, jp, ij, i, j, k;
  int point;

  for (ijp = 0; ijp < npcell; ijp++) {
    ip = ijp / npx;
    jp = ijp % npx;
    point = (pcell + ijp)->point;
    for (ij = 0; ij < cell_size * cell_size; ij++) {
      i = ij / cell_size;
      j = ij % cell_size;
      for (k = 0; k < 3; k++) {
	*(image->data + xm * 3 + ym * image->width * 3 + k +
	  j * 3 + i * image->width * 3 + jp * cell_size * 3 +
	  ip * image->width * cell_size * 3) =
	  *((pcell + point)->data + 3 * ij + k);
      }
    }
  }

  XDestroyImage (big_image);
  big_image = makeXimage (d, visual, depth, image);

  XPutImage (d, big_pmap, gc_big, big_image, 0, 0, 0, 0,
	     big_image->width, big_image->height);

  if (status & PUZZLE_WITH_BORDER) {
    for (ijp = 0; ijp < npcell; ijp++) {
      ip = ijp / npx;
      jp = ijp % npx;
      XDrawRectangle (d, big_pmap, gc_big,
		      xm + jp * cell_size,
		      ym + ip * cell_size, cell_size, cell_size);
    }
  }
  XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
	     big_image->width, big_image->height, 0, 0);
  puzzleScoreDisplay ();
}

int *
getRandomSequence (int n)
{
  char done;
  int *r, i, j;

  r = (int *) malloc (sizeof (int) * n);

  *r = random () / (RAND_MAX / n);

  for (i = 1; i < n; i++) {
    *(r + i) = random () / (RAND_MAX / n);
    done = FALSE;
    while (!done) {
      for (j = 0; j < i; j++) {
	if (*(r + i) == *(r + j)) {
	  *(r + i) = random () / (RAND_MAX / n);
	  break;
	}
      }
      if (j == i)
	done = TRUE;
    }
  }
  return r;
}

int
getPuzzlePoint (x, y)
{
  int point;

  point = (x - xm) / cell_size + npx * ((y - ym) / cell_size);

  if (npcell - 1 < point)
    return -1;
  else
    return point;
}

void
handlePuzzleEvent (XEvent * event)
{
  extern int depth;
  extern int thumb_offset, xwa_thumb_width, xwa_thumb_height;
  extern Display *d;
  extern Window big_window, thumb_window;
  extern XImage *big_image;
  extern GC gc_big;
  extern Pixmap big_pmap, thumb_pmap;
  extern Visual *visual;


  static char moving = FALSE;
  static int xsp0, ysp0, xsp, ysp, xep, yep;
  static int selected;
  static IMG img;
  static XImage *xim;

  char kbuff[16];
  int point, dest;
  KeySym ks;

  switch (event->type) {

  case Expose:
    if (event->xexpose.count == 0) {
      XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
		 big_image->width, big_image->height, 0, 0);
      XCopyArea (d, thumb_pmap, thumb_window, gc_big, 0,
		 -thumb_offset, xwa_thumb_width, xwa_thumb_height, 0, 0);
    }
    break;

  case ButtonPress:
    if (event->xbutton.button == Button1) {
      score--;
      puzzleHelpReturn ();

      xsp = event->xbutton.x;
      ysp = event->xbutton.y;
      xsp0 = (xsp - xm) % cell_size;
      ysp0 = (ysp - ym) % cell_size;
      selected = getPuzzlePoint (xsp, ysp);
      point = (pcell + selected)->point;
      XFillRectangle (d, big_pmap, gc_big,
		      xsp - xsp0, ysp - ysp0, cell_size, cell_size);
      /*
         XFillRectangle(d, big_window, gc_big,
         xsp-xsp0, ysp-ysp0, cell_size, cell_size);
       */
      img.width = cell_size;
      img.height = cell_size;
      img.data = (JSAMPLE *) (pcell + point)->data;
      xim = makeXimage (d, visual, depth, &img);
      small_pmap = XCreatePixmap (d, big_window, cell_size, cell_size, depth);
      exist_small_pmap = TRUE;
      XPutImage (d, small_pmap, gc_big, xim, 0, 0, 0, 0, cell_size,
		 cell_size);
      XDestroyImage (xim);
      moving = TRUE;
    }
    break;

  case MotionNotify:
    if (moving) {
      xep = event->xbutton.x;
      yep = event->xbutton.y;
      XCopyArea (d, big_pmap, big_window, gc_big,
		 xsp - xsp0, ysp - ysp0, cell_size, cell_size,
		 xsp - xsp0, ysp - ysp0);
      XCopyArea (d, small_pmap, big_window, gc_big, 0, 0,
		 cell_size, cell_size, xep - xsp0, yep - ysp0);
      xsp = xep;
      ysp = yep;
    }
    break;

  case ButtonRelease:
    if (!moving)
      break;
    dest = getPuzzlePoint (event->xbutton.x, event->xbutton.y);

    if (-1 < selected && -1 < dest && selected != dest) {
      swap (&((pcell + selected)->point), &((pcell + dest)->point));
    }
    rehashPuzzleState ();
    moving = FALSE;
    checkPuzzleState ();
    break;

  case KeyPress:
    XLookupString ((XKeyEvent *) event, kbuff, sizeof (kbuff),
		   &ks, (XComposeStatus *) NULL);

    handleControlKey (event);

    if (XEventsQueued (d, QueuedAfterReading) > 1)
      break;

    puzzleHelpReturn ();

    if (handlePuzzleKey (ks) == EXIT_PUZZLE) {
      if (exist_small_pmap) {
	XFreePixmap (d, small_pmap);
	exist_small_pmap = FALSE;
      }
      killBigWindow ();
    }
    break;

  case KeyRelease:
    handleControlKey (event);
    checkPuzzleState ();
    break;

  case ClientMessage:
    {
      Atom delw;
      XClientMessageEvent *client_event;

      client_event = (XClientMessageEvent *) event;
      delw = XInternAtom (d, "WM_DELETE_WINDOW", FALSE);

      if (client_event->data.l[0] == delw) {
	if (exist_small_pmap) {
	  XFreePixmap (d, small_pmap);
	  exist_small_pmap = FALSE;
	}
	killBigWindow ();
	return;
      }
    }
    break;

  default:
    ;
  }
}

void
checkPuzzleState (void)
{
  extern Display *d;
  extern IMG *image;

  int i, n;

  n = (image->width / cell_size) * (image->height / cell_size);

  for (i = 0; i < n; i++) {
    if ((pcell + i)->point != i)
      return;
  }
  puzzleComplete ();

  freePcell ();
  status &= ~EXIST_PUZZLE;

  if (exist_small_pmap) {
    XFreePixmap (d, small_pmap);
    exist_small_pmap = FALSE;
  }

  return;
}

void
freePcell (void)
{
  int i;

  for (i = 0; i < npcell; i++) {
    free ((pcell + i)->data);
  }
  free (pcell);

  free (tmp_sequence);
}

void
handleControlKey (XEvent * event)
{
  char kbuff[16];
  KeySym ks;

  XLookupString ((XKeyEvent *) event, kbuff, sizeof (kbuff),
		 &ks, (XComposeStatus *) NULL);

  if (ks == XK_Control_L || ks == XK_Control_R ||
      ks == XK_Shift_L || ks == XK_Shift_R ||
      ks == XK_Alt_L || ks == XK_Alt_R) {
    if (event->type == KeyPress)
      status |= CONTROL_STATE;
    else if (event->type == KeyRelease)
      status &= ~CONTROL_STATE;
  }
}
int
handlePuzzleKey (KeySym ks)
{
  switch (ks) {
  case XK_H:
  case XK_Help:
    score -= 10;
    puzzleHelp ();
    break;

  case XK_k:
  case XK_Up:
    if (status & CONTROL_STATE)
      verticalBackwardCell ();
    else
      upLineCell ();
    break;

  case XK_j:
  case XK_Down:
    if (status & CONTROL_STATE)
      verticalForwardCell ();
    else
      downLineCell ();
    break;

  case XK_l:
  case XK_Left:
    if (status & CONTROL_STATE)
      horizontalBackwardCell ();
    else
      leftLineCell ();
    break;

  case XK_h:
  case XK_Right:
    if (status & CONTROL_STATE)
      horizontalForwardCell ();
    else
      rightLineCell ();
    break;

  case XK_s:
    score -= 10;
    solvePuzzleOneStroke ();
    break;

  case XK_q:
  case XK_Q:
    return EXIT_PUZZLE;
    break;
  default:
    return 0;
  }
  rehashPuzzleState ();
  return 0;
}

void
downLineCell (void)
{
  int i;

  for (i = 0; i < npcell; i++)
    *(tmp_sequence + i) = (pcell + i)->point;

  for (i = 0; i < npx; i++)
    (pcell + i)->point = *(tmp_sequence + i - npx + npcell);
  for (i = npx; i < npcell; i++)
    (pcell + i)->point = *(tmp_sequence + i - npx);
}

void
upLineCell (void)
{
  int i;

  for (i = 0; i < npcell; i++)
    *(tmp_sequence + i) = (pcell + i)->point;

  for (i = 0; i < npcell - npx; i++)
    (pcell + i)->point = *(tmp_sequence + i + npx);
  for (i = npcell - npx; i < npcell; i++)


    (pcell + i)->point = *(tmp_sequence + i + npx - npcell);
}

void
rightLineCell (void)
{
  int i;

  for (i = 0; i < npcell; i++)
    *(tmp_sequence + i) = (pcell + i)->point;

  for (i = 1; i < npcell; i++)
    (pcell + i)->point = *(tmp_sequence + i - 1);
  for (i = 0; i < npy; i++)


    (pcell + i * npx)->point = *(tmp_sequence + i * npx + npx - 1);

}

void
leftLineCell (void)
{
  int i;

  for (i = 0; i < npcell; i++) {
    *(tmp_sequence + i) = (pcell + i)->point;
  }
  for (i = 0; i < npcell - 1; i++)
    (pcell + i)->point = *(tmp_sequence + i + 1);
  for (i = 0; i < npy; i++)

    (pcell + i * npx + npx - 1)->point = *(tmp_sequence + i * npx);
}

void
solvePuzzleOneStroke (void)
{
  int i, p1, p2;

  for (i = 0; i < npcell; i++) {
    if ((pcell + i)->point != i)
      break;
  }
  p1 = i;

  for (i = 0; i < npcell; i++) {
    if ((pcell + i)->point == p1)
      break;
  }

  p2 = i;

  swap (&((pcell + p1)->point), &((pcell + p2)->point));
}

void
horizontalForwardCell (void)
{
  int i, tmp;

  tmp = (pcell + npcell - 1)->point;
  for (i = npcell - 1; 0 < i; i--) {
    (pcell + i)->point = (pcell + i - 1)->point;
  }
  pcell->point = tmp;

}

void
horizontalBackwardCell (void)
{
  int i, tmp;

  tmp = pcell->point;
  for (i = 0; i < npcell - 1; i++) {
    (pcell + i)->point = (pcell + i + 1)->point;
  }
  (pcell + npcell - 1)->point = tmp;
}

void
verticalForwardCell (void)
{
  int i;

  for (i = 0; i < npcell; i++) {
    *(tmp_sequence + i) = (pcell + i)->point;
  }
  pcell->point = *(tmp_sequence + npcell - 1);
  for (i = 1; i < npx; i++) {

    (pcell + i)->point = *(tmp_sequence + i + npcell - npx - 1);
  }

  for (i = npx; i < npcell; i++) {
    (pcell + i)->point = *(tmp_sequence + i - npx);
  }
}

void
verticalBackwardCell (void)
{
  int i;

  for (i = 0; i < npcell; i++) {
    *(tmp_sequence + i) = (pcell + i)->point;
  }
  for (i = 0; i < npcell - npx; i++) {
    (pcell + i)->point = *(tmp_sequence + i + npx);
  }

  for (i = npcell - npx; i < npcell - 1; i++) {

    (pcell + i)->point = *(tmp_sequence + i - npcell + npx + 1);
  }
  (pcell + npcell - 1)->point = *tmp_sequence;
}

void
puzzleHelp (void)
{
  int i, j;

  hint_sequence = (int *) malloc (sizeof (int) * npcell);
  for (i = 0; i < npcell; i++) {
    *(hint_sequence + i) = TRUE;

    if ((pcell + i)->point != i) {
      *(hint_sequence + i) = FALSE;

      if (status & VERBOSE)
	fprintf (stderr, PUZZLE_HELP, i);

      for (j = 0; j < cell_size * cell_size * 3; j++) {
	*(((pcell + i)->data) + j) = ~(*(((pcell + i)->data) + j));
      }
    }
  }
}
void
puzzleHelpReturn (void)
{
  if (hint_sequence) {
    int i, j;

    for (i = 0; i < npcell; i++) {
      if (!*(hint_sequence + i)) {
	for (j = 0; j < cell_size * cell_size * 3; j++)
	  *(((pcell + i)->data) + j) = ~(*(((pcell + i)->data) + j));
      }
    }
    free (hint_sequence);
    hint_sequence = NULL;
  }
}

void
puzzleScoreDisplay (void)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_clip, gc_big;
  extern Pixmap big_pmap;
  extern XImage *big_image;

  char buff[128];

  XFillRectangle (d, big_pmap, gc_big, 0, 0, 105, 20);
  sprintf (buff, "score: %d/%d", score, 4 * npcell);

  XDrawString (d, big_pmap, gc_clip, 5, 15, buff, strlen (buff));

  XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
	     big_image->width, big_image->height, 0, 0);
  XFlush (d);

}
void
puzzleComplete (void)
{
  extern Display *d;
  extern Window big_window;
  extern GC gc_clip, gc_big;
  extern Pixmap big_pmap;
  extern XImage *big_image;

  XFillRectangle (d, big_pmap, gc_big, 0, 20, 105, 20);

  if (score == 4 * npcell)

    XDrawString (d, big_pmap, gc_clip, 5, 35, "PERFECT GAME!!", 14);
  else
    XDrawString (d, big_pmap, gc_clip, 5, 35, "Completed!", 10);

  XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
	     big_image->width, big_image->height, 0, 0);
  XFlush (d);
}
