#include "lsjpg.h"

static int last_line_height;
static int thumb_vscroll_y1, thumb_vscroll_y2;
static GC gc_thumb, gc_vscroll;
static Pixmap vscroll_pmap;
static int previous_image = -1;

void
handleThumbWinEvent (XEvent * event)
{
  extern unsigned status;
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];
  extern int xwa_thumb_width, xwa_thumb_height;
  extern int selected_image, n_image, thumb_offset, border_width;
  extern int pos[MAX_IMAGE][2];
  extern Window thumb_window;
  extern Pixmap thumb_pmap;
  extern Display *d;

  char *get_current_dir_name (void);
  XClientMessageEvent *client_event;
  /*  XReparentEvent *reparent_event;*/
  XWindowChanges xwc;
  Atom proto, delw;

  switch (event->type) {
  case Expose:
    if (event->xexpose.count == 0)
      XCopyArea (d, thumb_pmap, thumb_window, gc_thumb,
		 0, -thumb_offset, xwa_thumb_width, xwa_thumb_height, 0, 0);
    break;

  case ClientMessage:
    client_event = (XClientMessageEvent *) event;
    proto = XInternAtom (d, "WM_PROTOCOLS", FALSE);
    delw = XInternAtom (d, "WM_DELETE_WINDOW", FALSE);
    if (client_event->message_type == proto &&
	client_event->data.l[0] == delw) exit (0);
    break;

    /*
  case ReparentNotify:
    reparent_event = (XReparentEvent *) event;
    fprintf (stderr, "reparent: %d, %d\n", 
	     reparent_event->x, reparent_event->y);
    break;
    */

  case ConfigureNotify:
#ifdef OPAQUE_RESIZING
    if (XEventsQueued (d, QueuedAfterReading) < 5)
      thumbConfig (event->xconfigure.width);
#else
    thumbConfig (event->xconfigure.width);
#endif
    break;

  case ButtonPress:
    if (event->xbutton.button == Button1) {
      selected_image = seekSelectedImage (event->xbutton.x, event->xbutton.y);
      if (selected_image == -1) {
	if (pos[n_image - 1][1] + last_line_height + border_width
	    < xwa_thumb_height) {
	  xwc.height = pos[n_image - 1][1] + last_line_height + border_width;
	  XConfigureWindow (d, thumb_window, CWHeight, &xwc);
	}
	break;
      }
      displaySelectedImage ();
    }
    else if (event->xbutton.button == Button2) {
      selected_image = seekSelectedImage (event->xbutton.x, event->xbutton.y);
      if (status & SELECTION_OUTPUT && selected_image != -1) {
	if (*dname[selected_image] == '/')
	  printf ("%s\n", dname[selected_image]);
	else
	  printf ("%s/%s\n", get_current_dir_name (), dname[selected_image]);
      }
    }
    else if (event->xbutton.button == Button3) {
      if (status & EXIST_BIG_WINDOW)
	killBigWindow ();
    }
    else if (event->xbutton.button == Button4) {
      scrollUp ();
    }
    else if (event->xbutton.button == Button5) {
      scrollDown ();
    }

    break;

  case KeyPress:
    handleThumbWinKey (event);
    break;

  default:
    ;
  }
}

void
thumbConfig (int width)
{
  extern int n_image, thumb_offset, border_width;
  extern int pos[MAX_IMAGE][2];
  extern int xwa_thumb_width, xwa_thumb_height;
  extern Window thumb_window, vscroll;
  extern Display *d;
  extern int depth;

  XWindowAttributes xwa_thumb, xwa_vscroll;
  XWindowChanges xwc;

  imageConfig (n_image, (int *) pos, width - SCROLLBAR_WIDTH);
  XGetWindowAttributes (d, thumb_window, &xwa_thumb);

  if (pos[n_image - 1][1] + last_line_height
      + border_width + thumb_offset < xwa_thumb.height || thumb_offset > 0)
    thumb_offset = 0;

  XGetWindowAttributes (d, vscroll, &xwa_vscroll);

  if (xwa_vscroll.x != xwa_thumb.width - SCROLLBAR_WIDTH ||
      xwa_vscroll.height != xwa_thumb.height) {
    xwc.x = xwa_thumb.width - SCROLLBAR_WIDTH;
    xwc.y = 0;
    xwc.width = SCROLLBAR_WIDTH;
    xwc.height = xwa_thumb.height;
    XConfigureWindow (d, vscroll, CWX | CWY | CWWidth | CWHeight, &xwc);
  }

  if (vscroll_pmap)
    XFreePixmap (d, vscroll_pmap);
  vscroll_pmap = XCreatePixmap (d, vscroll,
				xwa_vscroll.width, xwa_thumb.height, depth);
  xwa_thumb_width = xwa_thumb.width;
  xwa_thumb_height = xwa_thumb.height;

  vscrollConfig ();
}

int
seekSelectedImage (int x, int y)
{
  extern int n_image, pos[MAX_IMAGE][2], thumb_offset;
  extern XImage *thumb_image[MAX_IMAGE];

  int i;

  for (i = 0; i < n_image; i++) {
    if (pos[i][0] < x && x < pos[i][0] + thumb_image[i]->width)
      if (pos[i][1] + thumb_offset < y &&
	  y < pos[i][1] + thumb_offset + thumb_image[i]->height)
	return i;
  }
  return -1;
}

void
displaySelectedImage (void)
{
  extern unsigned status;
  extern char fname[MAX_IMAGE][FILENAME_LENGTH];
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];
  extern int selected_image, original_image_size;
  extern int xs0, ys0, xs, ys, xe, ye;
  extern GC gc_big, gc_clip, gc_paint;
  extern IMG *image;
  extern Visual *visual;
  extern XImage *big_image;
  extern Pixmap big_pmap;
  extern Window big_window, thumb_window;
  extern Display *d;
  extern int depth;

  Window child;
  Atom delw;
  XSizeHints xsh;
  XWindowChanges xwc;
  XWindowAttributes xwa;

  if (status & EXIST_PUZZLE)
    freePcell ();
  status &= ~EXIST_PUZZLE;
  xs0 = ys0 = xs = ys = xe = ye = 0;
  status &= ~EXIST_CLIP;

  if (!(status & EXIST_BIG_WINDOW)) {
    XGCValues gcv;
    int x0, y0;

    image = (IMG *) malloc (sizeof (IMG));
    readJPEG (dname[selected_image], image);
    big_image = makeXimage (d, visual, depth, image);

    if (status & KEEP_POSITION) {
      x0 = 0;
      y0 = 0;
    }
    else {
      XGetWindowAttributes (d, thumb_window, &xwa);
      XTranslateCoordinates (d, thumb_window, RootWindow (d, 0),
			     0, 0, &xwa.x, &xwa.y, &child);
      x0 = xwa.x + xwa.width;
      y0 = xwa.y;
    }

    big_window = XCreateSimpleWindow (d, RootWindow (d, 0), x0, y0,
				      big_image->width, big_image->height,
				      0, BlackPixel (d, 0), BlackPixel (d,
									0));
    if (status & KEEP_POSITION) {
      xsh.x = x0;
      xsh.y = y0;
      xsh.flags = USPosition;
      XSetWMNormalHints (d, big_window, &xsh);
    }
    delw = XInternAtom (d, "WM_DELETE_WINDOW", False);
    XSetWMProtocols (d, big_window, &delw, 1);
    gc_big = XCreateGC (d, big_window, 0, 0);
    gc_paint = XCreateGC (d, big_window, 0, 0);
    XDefineCursor (d, big_window, XCreateFontCursor (d, XC_left_ptr));

    gcv.foreground = BlackPixel (d, 0) ^ WhitePixel (d, 0);
    gcv.background = WhitePixel (d, 0);
    gcv.function = GXxor;
    gc_clip = XCreateGC (d, big_window,
			 GCForeground | GCBackground | GCFunction, &gcv);

    XSelectInput (d, big_window,
		  /* StructureNotifyMask | ExposureMask | ButtonPressMask |*/
		  ExposureMask | ButtonPressMask |
		  KeyPressMask | KeyReleaseMask |
		  ButtonReleaseMask | ButtonMotionMask);

    XMapWindow (d, big_window);
  }
  else {	/* EXIST_BIG_WINDOW, change Big-widnow size */
    free (image->data);
    free (image);
    free (big_image->data);
    free (big_image);
    image = (IMG *) malloc (sizeof (IMG));
    readJPEG (dname[selected_image], image);
    big_image = makeXimage (d, visual, depth, image);

    if (status & KEEP_POSITION) {
      xwc.x = 0;
      xwc.y = 0;
    } else {
      XGetWindowAttributes (d, big_window, &xwa);
      XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			     0, 0, &xwa.x, &xwa.y, &child);
      xwc.x = xwa.x;
      xwc.y = xwa.y;
    }
    xwc.width = big_image->width;
    xwc.height = big_image->height;
    /*
    XConfigureWindow (d, big_window, CWX | CWY | CWWidth | CWHeight, &xwc);
    */
    XConfigureWindow (d, big_window, CWWidth | CWHeight, &xwc);
  }

  XStoreName (d, big_window, (char *) (fname + selected_image));
  if (!(status & LIGHT_ACTION)) {
    char buff[FILENAME_LENGTH];
    original_image_size = big_image->height;
    sprintf (buff, "%s: %dx%d(%d%%)",
	     (char *) (fname + selected_image),
	     big_image->width, big_image->height,
	     (int) (100 * ((float) big_image->height /
			   original_image_size) + 0.5));
    XStoreName (d, big_window, buff);
    XSetIconName (d, big_window, (char *) (fname + selected_image));
  }

  if (big_pmap)
    XFreePixmap (d, big_pmap);

  big_pmap = XCreatePixmap (d, big_window,
			    big_image->width, big_image->height, depth);

  XPutImage (d, big_pmap, gc_big, big_image, 0, 0, 0,
	     0, big_image->width, big_image->height);
  XCopyArea (d, big_pmap, big_window, gc_big, 0, 0,
	     big_image->width, big_image->height, 0, 0);
  status |= EXIST_BIG_WINDOW;

  if (!(status & LIGHT_ACTION)) {
    markSelectedImage (selected_image, previous_image);
    previous_image = selected_image;
    makeIcon ();
  }

  XMapRaised (d, big_window);
}

void
markSelectedImage (int select, int prev)
{
  extern Display *d;
  extern Pixmap thumb_pmap;
  extern Window thumb_window;
  extern XImage *thumb_image[MAX_IMAGE];
  extern int pos[MAX_IMAGE][2];
  extern int thumb_offset;

  int *p;
  XWindowAttributes xwa_thumb;

  XGetWindowAttributes (d, thumb_window, &xwa_thumb);

  p = (int *) pos;

  if (select == prev)
    return;

  if (prev != -1) {
    XPutImage (d, thumb_pmap, gc_thumb, thumb_image[prev], 0, 0,
	       *(p + 2 * prev), *(p + 2 * prev + 1),
	       thumb_image[prev]->width, thumb_image[prev]->height);
  }
  showSelectedRect (select);

  XCopyArea (d, thumb_pmap, thumb_window, gc_thumb, 0,
	     -thumb_offset, xwa_thumb.width, xwa_thumb.height, 0, 0);
  handleAutoScroll (&xwa_thumb);
}

void
showSelectedRect (int ip)
{
  extern Display *d;
  extern Window thumb_window;
  extern GC gc_clip;
  extern Pixmap thumb_pmap;
  extern XImage *thumb_image[MAX_IMAGE];
  extern int pos[MAX_IMAGE][2];

  int i;
  XWindowAttributes xwa_thumb;

  XGetWindowAttributes (d, thumb_window, &xwa_thumb);

  for (i = 0; i < 4; i++) {
    XDrawRectangle (d, thumb_pmap, gc_clip,
		    pos[ip][0] + i + 1,
		    pos[ip][1] + i + 1,
		    thumb_image[ip]->width - 2 * i - 1 - 2,
		    thumb_image[ip]->height - 2 * i - 1 - 2);
  }
}

void
handleAutoScroll (XWindowAttributes * xwa)
{
  extern int pos[MAX_IMAGE][2], selected_image;
  extern int n_image, thumb_offset, thumb_height, border_width;

  if (selected_image == 0) {
    thumb_offset = 0;
    rehashThumbWin (xwa);
    return;
  }

  if (selected_image == n_image - 1) {
    thumb_offset = -pos[selected_image][1] - border_width;
    rehashThumbWin (xwa);
    return;
  }

  /*  while (pos[selected_image][1] < -thumb_offset + border_width) { */
  while (pos[selected_image][1] < -thumb_offset) {
    thumb_offset += thumb_height + border_width;
    rehashThumbWin (xwa);
  }

  while (xwa->height - thumb_offset <
	 pos[selected_image][1] + thumb_height + border_width) {
    int tmp = thumb_offset;

    thumb_offset -= thumb_height + border_width;
    rehashThumbWin (xwa);
    if (tmp == thumb_offset)	/* rehahThumbWin changes thumb_offset */
      break;
  }
}

void
handleVscroll (XEvent * event)
{
  extern unsigned status;
  extern int ys, ye;
  extern int thumb_offset, n_image, border_width;
  extern int pos[MAX_IMAGE][2];
  extern Display *d;
  extern Window thumb_window, vscroll;
  extern Pixmap thumb_pmap;

  XWindowAttributes xwa;
  int height;

  switch (event->type) {
  case Expose:
    XGetWindowAttributes (d, vscroll, &xwa);
    XCopyArea (d, vscroll_pmap, vscroll,
	       gc_vscroll, 0, 0, xwa.width, xwa.height, 0, 0);
    break;

  case ButtonPress:
    if (event->xbutton.y < thumb_vscroll_y1) {
      while (1) {
	if (XEventsQueued (d, QueuedAfterReading) < 1)
	  scrollUp ();
	XNextEvent (d, event);
	if (event->type == ButtonRelease)
	  break;
      }
    }
    else if (event->xbutton.y > thumb_vscroll_y1 + thumb_vscroll_y2) {
      while (1) {
	if (XEventsQueued (d, QueuedAfterReading) < 1)
	  scrollDown ();
	XNextEvent (d, event);
	if (event->type == ButtonRelease)
	  break;
      }
    }
    else {
      ys = event->xbutton.y;
      status |= VSCROLLING;
    }
    break;

  case ButtonRelease:
    status &= ~VSCROLLING;
    break;

  case MotionNotify:
    if (status & VSCROLLING) {
      ye = event->xbutton.y;

      XGetWindowAttributes (d, thumb_window, &xwa);
      height = pos[n_image - 1][1] + last_line_height + border_width;
      thumb_offset -= (ye - ys) * (float) height / xwa.height;

      if (height + thumb_offset < xwa.height)
	thumb_offset = -height + xwa.height;
      if (thumb_offset > 0)
	thumb_offset = 0;

      XCopyArea (d, thumb_pmap, thumb_window,
		 gc_thumb, 0, -thumb_offset, xwa.width, xwa.height, 0, 0);
      vscrollConfig ();
      ys = ye;
    }
    break;

  default:
    ;
  }
}

void
scrollDown (void)
{
  extern int thumb_offset, thumb_height, n_image, border_width;
  extern int pos[MAX_IMAGE][2];
  extern Display *d;
  extern Window thumb_window;
  extern Pixmap thumb_pmap;

  XWindowAttributes xwa;
  int height;

  XGetWindowAttributes (d, thumb_window, &xwa);
  height = pos[n_image - 1][1] + last_line_height + border_width;

  thumb_offset -= thumb_height + border_width;
  if (height + thumb_offset < xwa.height)
    thumb_offset = -height + xwa.height;
  if (thumb_offset > 0)
    thumb_offset = 0;

  XCopyArea (d, thumb_pmap, thumb_window, gc_thumb, 0, -thumb_offset,
	     xwa.width, xwa.height, 0, 0);
  vscrollConfig ();
}

void
scrollUp (void)
{
  extern int thumb_offset, thumb_height, border_width;
  extern Display *d;
  extern Window thumb_window;
  extern Pixmap thumb_pmap;

  XWindowAttributes xwa;

  XGetWindowAttributes (d, thumb_window, &xwa);
  thumb_offset += thumb_height + border_width;
  if (thumb_offset > 0)
    thumb_offset = 0;
  XCopyArea (d, thumb_pmap, thumb_window, gc_thumb,
	     0, -thumb_offset, xwa.width, xwa.height, 0, 0);
  vscrollConfig ();
}

void
imageConfig (int n_image, int *pos, int width)
{
  extern int border_width;
  extern XImage *thumb_image[MAX_IMAGE];
  extern Pixmap thumb_pmap;
  extern Display *d;
  extern Window thumb_window;

  unsigned int width_tmp = 0, height_tmp = border_width, h_wrk, height;
  int i, j, jp = 0;
  XWindowAttributes xwa;

  for (i = 0; i < n_image; i++) {
    width_tmp += thumb_image[i]->width + border_width;
    if (width_tmp > width - border_width) {
      h_wrk = 0;
      for (j = jp; j < i; j++) {
	if (h_wrk < thumb_image[j]->height)
	  h_wrk = thumb_image[j]->height;
	*(pos + 2 * j) += (width - (width_tmp - thumb_image[i]->width)) / 2;
      }
      jp = i;
      height_tmp += h_wrk + border_width;
      width_tmp = thumb_image[i]->width + border_width;
      *(pos + 2 * i + 0) = border_width;
    }
    else {
      *(pos + 2 * i + 0) = width_tmp - thumb_image[i]->width;
    }
    *(pos + 2 * i + 1) = height_tmp;
  }

  last_line_height = 0;
  for (j = jp; j < i; j++) {
    if (last_line_height < thumb_image[j]->height)
      last_line_height = thumb_image[j]->height;
    *(pos + 2 * j) += (width - width_tmp) / 2;
  }

  height = *(pos + 2 * (n_image - 1) + 1) + last_line_height + border_width;

  createThumbPixmap (width, height);

  XGetWindowAttributes (d, thumb_window, &xwa);
  XFillRectangle (d, thumb_pmap, gc_thumb, 0, 0, width,
		  *(pos + 2 * (n_image - 1) + 1)
		  + last_line_height + border_width);

  for (i = 0; i < n_image; i++)
    XPutImage (d, thumb_pmap, gc_thumb, thumb_image[i], 0, 0,
	       *(pos + 2 * i), *(pos + 2 * i + 1),
	       thumb_image[i]->width, thumb_image[i]->height);
}

void
createThumbPixmap (int width, int height)
{
  static char said_once = FALSE;

  extern int depth;
  extern Pixmap thumb_pmap;
  extern Display *d;
  extern Window thumb_window;

  if (said_once == FALSE && 32768 < height) {
    fprintf (stderr,
	     "createThumbPixmap(thumbev.c): creating thumb_pmap %d x %d\n",
	     (int) width, (int) height);
    fprintf (stderr,
	     "createThumbPixmap(thumbev.c): 32768(=2**16/2) < height\n");
    fprintf (stderr,
	     "Program could not create pixmap of large height. You should resize window wider to avoid this. Ctrl + \"Right arow\" key helps you.\n");
    said_once = TRUE;
  }

  if (thumb_pmap)
    XFreePixmap (d, thumb_pmap);

  thumb_pmap = XCreatePixmap (d, thumb_window, width, height, depth);

  if (!thumb_pmap)
    fprintf (stderr, "createThumbPixmaq: falied to create thumb_pmap\n");
}

void
vscrollConfig (void)
{
  extern int n_image, thumb_offset, pos[MAX_IMAGE][2];
  extern int border_width, xwa_thumb_width, xwa_thumb_height;
  extern Window vscroll;
  extern Display *d;

  int height;

  XFillRectangle (d, vscroll_pmap, gc_vscroll,
		  0, 0, xwa_thumb_width, xwa_thumb_height);
  height = pos[n_image - 1][1] + last_line_height + border_width;
  thumb_vscroll_y1 = xwa_thumb_height * (float) (-thumb_offset) / height;
  thumb_vscroll_y2 = xwa_thumb_height * (float) (xwa_thumb_height) / height;

  XFillRectangle (d, vscroll_pmap, gc_thumb, 1, thumb_vscroll_y1,
		  SCROLLBAR_WIDTH - 3, thumb_vscroll_y2);
  XCopyArea (d, vscroll_pmap, vscroll, gc_vscroll,
	     0, 0, xwa_thumb_width, xwa_thumb_height, 0, 0);
}

void
handleThumbWinKey (XEvent * event)
{
  extern unsigned status;
  extern int thumb_offset, border_width, n_image;
  extern int pos[MAX_IMAGE][2];
  extern Window thumb_window;
  extern Display *d;

  char kbuff[16];

  KeySym ks;
  XWindowAttributes xwa_thumb;

  if (XEventsQueued (d, QueuedAfterReading) > 5)
    return;

  XLookupString ((XKeyEvent *)
		 event, kbuff, sizeof (kbuff), &ks, (XComposeStatus *) NULL);
  XGetWindowAttributes (d, thumb_window, &xwa_thumb);

  if (status & CONTROL_STATE) {
    thumbWinControlKey (ks, &xwa_thumb);
    return;
  }

  switch (ks) {
  case XK_Up:
  case XK_U:
  case XK_u:
    /*
    if (thumb_offset == 0)
      break;
    thumb_offset += thumb_height + border_width;
    rehashThumbWin (&xwa_thumb);
    */
    scrollUp ();
    break;

  case XK_Down:
  case XK_D:
  case XK_d:
    /*
    thumb_offset -= thumb_height + border_width;
    rehashThumbWin (&xwa_thumb);
    */
    scrollDown ();
    break;

  case XK_Page_Up:
    if (thumb_offset == 0)
      break;
    thumb_offset += xwa_thumb.height;
    rehashThumbWin (&xwa_thumb);
    break;

  case XK_Page_Down:
    thumb_offset -= xwa_thumb.height;
    rehashThumbWin (&xwa_thumb);
    break;

  case XK_Home:
    thumb_offset = 0;
    rehashThumbWin (&xwa_thumb);
    break;

  case XK_End:
    thumb_offset = -pos[n_image - 1][1] - border_width;
    rehashThumbWin (&xwa_thumb);
    break;

  default:
    ;
  }
}

void
thumbWinControlKey (int ks, XWindowAttributes * xwa)
{
  extern int thumb_height, border_width, thumb_offset;
  extern int pos[MAX_IMAGE][2], n_image;
  extern Window thumb_window;
  extern Display *d;

  XWindowChanges xwc;
  int height;

  switch (ks) {

  case XK_U:
  case XK_u:
    if (thumb_offset == 0)
      break;
    thumb_offset += xwa->height;
    rehashThumbWin (xwa);
    break;

  case XK_D:
  case XK_d:
    height = pos[n_image - 1][1] + last_line_height + border_width;
    thumb_offset -= xwa->height;

    if (height + thumb_offset < xwa->height)
      thumb_offset = -height + xwa->height;
    if (thumb_offset > 0)
      thumb_offset = 0;
    rehashThumbWin (xwa);
    break;

  case XK_Up:
    if (xwa->height > 2 * thumb_height) {
      xwc.height = xwa->height - thumb_height - border_width;
      XConfigureWindow (d, thumb_window, CWHeight, &xwc);
    }
    break;

  case XK_Down:
    xwc.height = xwa->height + thumb_height + border_width;
    XConfigureWindow (d, thumb_window, CWHeight, &xwc);
    break;

  case XK_Left:
    if (xwa->width > 2 * thumb_height) {
      xwc.width = xwa->width - thumb_height;
      XConfigureWindow (d, thumb_window, CWWidth, &xwc);
    }
    break;

  case XK_Right:
    xwc.width = xwa->width + thumb_height;
    XConfigureWindow (d, thumb_window, CWWidth, &xwc);
    break;

  default:
    ;
  }
}

void
makeIcon ()
{
  extern IMG *image;
  extern Window big_window;
  extern GC gc_big;
  extern int depth;
  extern Visual *visual;
  extern Display *d;

  Pixmap icon;
  XWMHints xwmh;
  XImage *icon_image;

  icon_image = makeXimage (d, visual, depth, resizeImage (image, 48));

  icon = XCreatePixmap (d, RootWindow (d, 0),
			icon_image->width, icon_image->height, depth);

  XPutImage (d, icon, gc_big, icon_image, 0, 0, 0, 0,
	     icon_image->width, icon_image->height);

  xwmh.icon_pixmap = icon;
  xwmh.flags = IconPixmapHint;

  XSetWMHints (d, big_window, &xwmh);
  free (icon_image->data);
  free (icon_image);
}

void
rehashThumbWin (XWindowAttributes * xwa)
{
  extern int thumb_offset, border_width, n_image;
  extern int pos[MAX_IMAGE][2];
  extern Window thumb_window;
  extern Pixmap thumb_pmap;
  extern Display *d;

  int height;

  height = pos[n_image - 1][1] + last_line_height + border_width;

  if (height + thumb_offset < xwa->height)
    thumb_offset = -height + xwa->height;
  if (thumb_offset > 0)
    thumb_offset = 0;

  XCopyArea (d, thumb_pmap, thumb_window, gc_thumb,
	     0, -thumb_offset, xwa->width, xwa->height, 0, 0);
  vscrollConfig ();
}
void
prepareThumbwindow (void)
{
  extern unsigned status;
  extern int xwa_thumb_width, xwa_thumb_height, n_image, thumb_offset;
  extern int border_width, pos[MAX_IMAGE][2];
  extern Pixmap thumb_pmap;
  extern Window thumb_window, vscroll;
  extern Display *d;
  extern int depth;

  char buff[FILENAME_LENGTH];
  char specify_geometry = FALSE;
  char *bgcolor, *xgeometry;
  int x0 = 0, y0 = 0, i;
  unsigned int width, height;
  Atom delw;
  XColor color;
  Colormap default_cmap;
  unsigned long bgpix, GC_mask;
  XGCValues xGCv;

  bgcolor = getThumbwinColor ();
  xgeometry = getThumbwinGeometry ();

  default_cmap = DefaultColormap (d, DefaultScreen (d));
  if (XParseColor (d, default_cmap, bgcolor, &color) == 0 ||
      XAllocColor (d, default_cmap, &color) == 0) {
    fprintf (stderr,
	     "lsjpg: prepareThumbwindow: Unknown Background color <%s>\n",
	     bgcolor);
    bgpix = BlackPixel (d, DefaultScreen (d));
  }
  else
    bgpix = color.pixel;

  i = XParseGeometry (xgeometry, &x0, &y0, &width, &height);
  if (i & XValue) {
    specify_geometry = TRUE;
    if (!(i & YValue))
      y0 = 0;
  }
  if (i & YValue) {
    specify_geometry = TRUE;
    if (!(i & XValue))
      x0 = 0;
  }
  if (!(i & WidthValue))
    width = DisplayWidth (d, DefaultScreen (d)) / 3;
  if (!(i & HeightValue))
    height = DisplayHeight (d, DefaultScreen (d)) - 2 * 64;

  if (status & KEEP_POSITION) {
    specify_geometry = TRUE;
    x0 = DisplayWidth (d, DefaultScreen (d)) - width - 12;
    y0 = 0;
  }
  thumb_window = XCreateSimpleWindow (d, RootWindow (d, 0),
				      x0, y0, width, height, 0,
				      WhitePixel (d, 0), bgpix);
  vscroll =
    XCreateSimpleWindow (d, thumb_window, width - SCROLLBAR_WIDTH, 0,
			 SCROLLBAR_WIDTH, height, 1, bgpix, ~bgpix);
  xwa_thumb_width = width;
  xwa_thumb_height = height;

  if (specify_geometry) {
    XSizeHints xsh;
    xsh.x = x0;
    xsh.y = y0;
    xsh.flags = USPosition;
    XSetWMNormalHints (d, thumb_window, &xsh);
  }
  delw = XInternAtom (d, "WM_DELETE_WINDOW", False);
  XSetWMProtocols (d, thumb_window, &delw, 1);

  XDefineCursor (d, thumb_window, XCreateFontCursor (d, XC_hand2));
  XDefineCursor (d, vscroll, XCreateFontCursor (d, XC_double_arrow));

  XSelectInput (d, thumb_window,
		ExposureMask | StructureNotifyMask | ButtonPressMask |
		KeyPressMask | KeyReleaseMask);
  XSelectInput (d, vscroll,
		ExposureMask | ButtonPressMask | ButtonReleaseMask |
		ButtonMotionMask);

  sprintf (buff, "%s(%dfiles)", VERSION, n_image);
  XStoreName (d, thumb_window, buff);
  XSetIconName (d, thumb_window, VERSION);

  GC_mask = GCForeground | GCBackground;
  xGCv.foreground = bgpix;
  xGCv.background = ~bgpix;
  gc_thumb = XCreateGC (d, thumb_window, GC_mask, &xGCv);
  xGCv.foreground = ~bgpix;
  xGCv.background = bgpix;
  gc_vscroll = XCreateGC (d, vscroll, GC_mask, &xGCv);

  imageConfig (n_image, (int *) pos, xwa_thumb_width - SCROLLBAR_WIDTH);

  if (vscroll_pmap)
    XFreePixmap (d, vscroll_pmap);
  vscroll_pmap = XCreatePixmap (d, vscroll, SCROLLBAR_WIDTH, height, depth);

  vscrollConfig ();
  XCopyArea (d, thumb_pmap, thumb_window, gc_thumb, 0,
	     -thumb_offset, xwa_thumb_width, xwa_thumb_height, 0, 0);

  if (pos[n_image - 1][1] + last_line_height + border_width
      < xwa_thumb_height) {
    XWindowChanges xwc;
    xwc.height = pos[n_image - 1][1] + last_line_height + border_width;
    XConfigureWindow (d, thumb_window, CWHeight, &xwc);
  }
  XMapWindow (d, thumb_window);
  XMapSubwindows (d, thumb_window);
  {
    XWMHints xwmh;
#include "lsjpg.xbm"
    xwmh.icon_pixmap =
      XCreatePixmapFromBitmapData (d, thumb_window, lsjpg_bits,
				   lsjpg_width, lsjpg_height, 1L, 0L, 1);
    xwmh.flags = IconPixmapHint;
    XSetWMHints (d, thumb_window, &xwmh);
  }
}

void
setSuitableWinPlace (void)
{
  extern unsigned status;
  extern int pos[MAX_IMAGE][2], n_image, border_width;
  extern Display *d;
  extern Window big_window, thumb_window;

  Window child;
  XWindowChanges xwc;
  XWindowAttributes xwa_thumb, xwa_big;
  int dispW = DisplayWidth (d, DefaultScreen (d));

  XGetWindowAttributes (d, thumb_window, &xwa_thumb);
  XTranslateCoordinates (d, thumb_window, RootWindow (d, 0),
			 0, 0, &xwa_thumb.x, &xwa_thumb.y, &child);

  if (xwa_thumb.height >
      pos[n_image - 1][1] + last_line_height + border_width) {
    xwc.height = pos[n_image - 1][1] + last_line_height + border_width;
    XConfigureWindow (d, thumb_window, CWHeight, &xwc);
  }

  if (status & EXIST_BIG_WINDOW) {
    XGetWindowAttributes (d, thumb_window, &xwa_thumb);
    XTranslateCoordinates (d, thumb_window, RootWindow (d, 0),
			   0, 0, &xwa_thumb.x, &xwa_thumb.y, &child);
    XGetWindowAttributes (d, big_window, &xwa_big);
    XTranslateCoordinates (d, big_window, RootWindow (d, 0),
			   0, 0, &xwa_big.x, &xwa_big.y, &child);

    if (xwa_thumb.x < dispW - xwa_thumb.x - xwa_thumb.width) {
      xwc.x = xwa_thumb.x + xwa_thumb.width;
      xwc.y = xwa_thumb.y;
    } else {
      xwc.x = xwa_thumb.x - xwa_big.width;
      xwc.y = xwa_thumb.y;
    }

    XConfigureWindow (d, big_window, CWX | CWY, &xwc);
  }
}

void
deleteImageFromList (int select)
{
  extern XImage *thumb_image[MAX_IMAGE];
  extern char fname[MAX_IMAGE][FILENAME_LENGTH];
  extern char dname[MAX_IMAGE][FILENAME_LENGTH];
  extern int pos[MAX_IMAGE][2], n_image, selected_image;

  int i;

  XDestroyImage (thumb_image[select]);

  for (i = select; i < n_image - 1; i++) {
    thumb_image[i] = thumb_image[i + 1];
    strcpy (fname[i], fname[i + 1]);
    strcpy (dname[i], dname[i + 1]);
    pos[i][0] = pos[i + 1][0];
    pos[i][1] = pos[i + 1][1];
  }

  n_image--;
  selected_image--;
  previous_image--;

  {
    extern Window thumb_window;
    extern Display *d;
    extern Pixmap thumb_pmap;
    extern int thumb_offset;
    XWindowAttributes xwa;

    XGetWindowAttributes (d, thumb_window, &xwa);
    thumbConfig (xwa.width);
    XCopyArea (d, thumb_pmap, thumb_window, gc_thumb,
	       0, -thumb_offset, xwa.width, xwa.height, 0, 0);
  }
}
