#include "lsjpg.h"

typedef unsigned char byte;
static int highbit (unsigned long);

static int
highbit (unsigned long ul)
{
  int i;
  unsigned long hb;

  hb = 0x80;
  hb = (hb << 24);

  for (i = 31; ((ul & hb) == 0) && i >= 0; i--, ul <<= 1);
  return i;
}

XImage *
makeXimage (Display * d, Visual * visual, int depth, IMG * img)
{
  byte *imagedata, *lip, *ip, *pp;
  int i, j;
  int rshift, gshift, bshift, bperpix, bperline, border;
  unsigned long r, g, b, rmask, gmask, bmask, xcol;
  XImage *ximage;

  rmask = visual->red_mask;
  gmask = visual->green_mask;
  bmask = visual->blue_mask;

  rshift = 7 - highbit (rmask);
  gshift = 7 - highbit (gmask);
  bshift = 7 - highbit (bmask);

  ximage = XCreateImage (d, visual, depth, ZPixmap, 0, NULL,
			 img->width, img->height, 32, 0);
  if (!ximage)
    fprintf (stderr, "lsjpg: ximage: XCreatrImage failed.\n");

  bperline = ximage->bytes_per_line;
  bperpix = ximage->bits_per_pixel;
  border = ximage->byte_order;

  imagedata = (byte *) malloc ((size_t) (img->height * bperline));
  if (!imagedata)
    fprintf (stderr, "lsjpg: makeXimage: Couldn't malloc imagedata\n");

  ximage->data = (char *) imagedata;

  if (bperpix != 8 && bperpix != 16 && bperpix != 24 && bperpix != 32) {
    fprintf (stderr,
	     "lsjpg: Sorry, no code written to handle %d-bit %s\n",
	     bperpix, "TrueColor/DirectColor displays!");
    exit (1);
  }
  lip = imagedata;
  pp = img->data;

  for (i = 0; i < img->height; i++, lip += bperline) {
    for (j = 0, ip = lip; j < img->width; j++) {

      r = *pp++;
      g = *pp++;
      b = *pp++;

      if (rshift < 0)
	r = r << (-rshift);
      else
	r = r >> rshift;

      if (gshift < 0)
	g = g << (-gshift);
      else
	g = g >> gshift;

      if (bshift < 0)
	b = b << (-bshift);
      else
	b = b >> bshift;

      r = r & rmask;
      g = g & gmask;
      b = b & bmask;
      xcol = r | g | b;

      if (bperpix == 16) {
	if (border == MSBFirst) {
	  *ip++ = (xcol >> 8) & 0xff;
	  *ip++ = xcol & 0xff;
	}
	else {
	  *ip++ = xcol & 0xff;
	  *ip++ = (xcol >> 8) & 0xff;
	}
      }
      else if (bperpix == 24) {
	if (border == MSBFirst) {
	  *ip++ = (xcol >> 16) & 0xff;
	  *ip++ = (xcol >> 8) & 0xff;
	  *ip++ = xcol & 0xff;
	}
	else {
	  *ip++ = xcol & 0xff;
	  *ip++ = (xcol >> 8) & 0xff;
	  *ip++ = (xcol >> 16) & 0xff;
	}
      }
      else if (bperpix == 32) {
	if (border == MSBFirst) {
	  *ip++ = (xcol >> 24) & 0xff;
	  *ip++ = (xcol >> 16) & 0xff;
	  *ip++ = (xcol >> 8) & 0xff;
	  *ip++ = xcol & 0xff;
	}
	else {
	  *ip++ = xcol & 0xff;
	  *ip++ = (xcol >> 8) & 0xff;
	  *ip++ = (xcol >> 16) & 0xff;
	  *ip++ = (xcol >> 24) & 0xff;
	}
      }
      else if (bperpix == 8) {
	*ip++ = (char) xcol & 0xff;
      }
    }
  }
  return ximage;
}
